<?php defined('SYSPATH') or die('No direct script access.');

// -- Environment setup --------------------------------------------------------

// Load the core Kohana class
require SYSPATH.'classes/Kohana/Core'.EXT;

if (is_file(APPPATH.'classes/Kohana'.EXT))
{
	// Application extends the core
	require APPPATH.'classes/Kohana'.EXT;
}
else
{
	// Load empty core extension
	require SYSPATH.'classes/Kohana'.EXT;
}

/**
 * Set the default time zone.
 *
 * @link http://kohanaframework.org/guide/using.configuration
 * @link http://www.php.net/manual/timezones
 */
date_default_timezone_set('Asia/Manila');

/**
 * Set the default locale.
 *
 * @link http://kohanaframework.org/guide/using.configuration
 * @link http://www.php.net/manual/function.setlocale
 */
setlocale(LC_ALL, 'en_US.utf-8');

/**
 * Enable the Kohana auto-loader.
 *
 * @link http://kohanaframework.org/guide/using.autoloading
 * @link http://www.php.net/manual/function.spl-autoload-register
 */
spl_autoload_register(array('Kohana', 'auto_load'));

/**
 * Optionally, you can enable a compatibility auto-loader for use with
 * older modules that have not been updated for PSR-0.
 *
 * It is recommended to not enable this unless absolutely necessary.
 */
//spl_autoload_register(array('Kohana', 'auto_load_lowercase'));

/**
 * Enable the Kohana auto-loader for unserialization.
 *
 * @link http://www.php.net/manual/function.spl-autoload-call
 * @link http://www.php.net/manual/var.configuration#unserialize-callback-func
 */
ini_set('unserialize_callback_func', 'spl_autoload_call');

/**
 * Set the mb_substitute_character to "none"
 *
 * @link http://www.php.net/manual/function.mb-substitute-character.php
 */
// mb_substitute_character('none');

// -- Configuration and initialization -----------------------------------------

/**
 * Set the default language
 */
I18n::lang('en-us');

if (isset($_SERVER['SERVER_PROTOCOL']))
{
	// Replace the default protocol.
	HTTP::$protocol = $_SERVER['SERVER_PROTOCOL'];
}

/**
 * Set Kohana::$environment if a 'KOHANA_ENV' environment variable has been supplied.
 *
 * Note: If you supply an invalid environment name, a PHP warning will be thrown
 * saying "Couldn't find constant Kohana::<INVALID_ENV_NAME>"
 */
if (isset($_SERVER['SERVER_NAME']))
{
	if ($_SERVER['SERVER_NAME'] !== 'localhost')
	{
	   // We are live!
	   Kohana::$environment = Kohana::PRODUCTION;

	   // Turn off notices and strict errors
	   error_reporting(E_ALL ^ E_NOTICE ^ E_STRICT);
	}
	else
	{
		Kohana::$environment = Kohana::PRODUCTION;
	}
}
else
{
	Kohana::$environment = Kohana::DEVELOPMENT;
}
/**
 * Initialize Kohana, setting the default options.
 *
 * The following options are available:
 *
 * - string   base_url    path, and optionally domain, of your application   NULL
 * - string   index_file  name of your index file, usually "index.php"       index.php
 * - string   charset     internal character set used for input and output   utf-8
 * - string   cache_dir   set the internal cache directory                   APPPATH/cache
 * - integer  cache_life  lifetime, in seconds, of items cached              60
 * - boolean  errors      enable or disable error handling                   TRUE
 * - boolean  profile     enable or disable internal profiling               TRUE
 * - boolean  caching     enable or disable internal caching                 FALSE
 * - boolean  expose      set the X-Powered-By header                        FALSE
 */
Kohana::init(array(
	'base_url'   => Kohana::$environment === Kohana::PRODUCTION ? '/eolsit2/' : '/eolsit2/',
	'caching'    => Kohana::$environment === Kohana::PRODUCTION,
	'profile'    => Kohana::$environment !== Kohana::PRODUCTION,
	'index_file' => '',
));

/**
 * Attach the file write to logging. Multiple writers are supported.
 */
Kohana::$log->attach(new Log_File(APPPATH.'logs'));

/**
 * Attach a file reader to config. Multiple readers are supported.
 */
Kohana::$config->attach(new Config_File);

/**
 * Enable modules. Modules are referenced by a relative or absolute path.
 */
Kohana::modules(array(
	// 'auth'       => MODPATH.'auth',       	// Basic authentication
	// 'cache'      => MODPATH.'cache',      	// Caching with multiple backends
	// 'codebench'  => MODPATH.'codebench',  	// Benchmarking tool
	'kohana-mssql'  => MODPATH.'kohana-mssql',	// MS SQL
	'database'  	 => MODPATH.'database',   	// Database access
	// 'image'      => MODPATH.'image',      	// Image manipulation
	'minion'     	 => MODPATH.'minion',     	// CLI Tasks
	'orm'        	 => MODPATH.'orm',        	// Object Relationship Mapping
	// 'unittest'   => MODPATH.'unittest',   	// Unit testing
	// 'userguide'  => MODPATH.'userguide',  	// User guide and API documentation
	'pagination'    => MODPATH.'pagination',   	// Pagination Module
	'swiftmailer'   => MODPATH.'swiftmailer',  	// Mailer
	'captcha'     	 => MODPATH.'captcha',  		// Captcha
	));


// Cookie::$salt = '$1$Testing Environment For MsSQL';
Cookie::$salt = '$2y$07$9150fmtqzcpQB1PGBtpeLbKbSs0nD9cwWunn5ZohnVm0bb0YbCX9BpfVEorsUyY';
Session::$default = 'database';

/**
 * Set the routes. Each route must have a minimum of a name, a URI and a set of
 * defaults for the URI.
 */
/*
Route::set('subject-form', 'page/subject_form')
	->defaults(array(
		'controller' => 'page',
		'action'     => 'subject_form',
	));
*/
Route::set('toplinks', 'admin/top_links(/<action>(/<id>(/<id2>)))')
	->defaults(array(
		'directory'  => 'admin',
		'controller' => 'links',
		'action'     => 'index',
	));

Route::set('admin', 'admin(/<controller>(/<action>(/<id>(/<id2>))))')
	->defaults(array(
		'directory'  => 'admin',
		'controller' => 'login',
		'action'     => 'index',
	));

Route::set('layout-preview', 'layout_preview/<controlno>(/<blurb>)')
	->defaults(array(
		'controller' => 'preview',
		'action'     => 'layout_preview',
	));

Route::set('preview2', 'preview2/<controlno>(/<blurb>)')
	->defaults(array(
		'controller' => 'preview',
		'action'     => 'page',
	));

Route::set('preview', 'preview/new_page')
	->defaults(array(
		'controller' => 'preview',
		'action'     => 'new_page',
	));

Route::set('p', 'p/<level>/<categ_id>(/<blurb>)')
	->defaults(array(
		'controller' => 'cms',
		'action'     => 'view',
	));

Route::set('cms-page', 'cms/page/<id>(/<blurb>)')
	->defaults(array(
		'controller' => 'cms',
		'action'     => 'page',
	));

Route::set('promos', 'promos(/<action>(/<id>(/<id2>)))')
	->defaults(array(
		'controller' => 'cms',
		'action'     => 'promo_index',
	));

Route::set('page', 'page(/<action>(/<id>))')
	->defaults(array(
		'controller' => 'page',
		'action'     => 'subject_form',
	));

Route::set('contact-us', 'contact-us')
	->defaults(array(
		'controller' => 'page',
		'action'     => 'contact_us',
	));

Route::set('toll-free-number', 'toll-free-number')
	->defaults(array(
		'controller' => 'page',
		'action'     => 'toll_free_number',
	));

Route::set('send-a-message', 'send-a-message')
	->defaults(array(
		'controller' => 'page',
		'action'     => 'send_a_message',
	));

Route::set('message-sent', 'message-sent')
	->defaults(array(
		'controller' => 'page',
		'action'     => 'message_sent',
	));

Route::set('message-not-sent', 'message-not-sent')
	->defaults(array(
		'controller' => 'page',
		'action'     => 'message_not_sent',
	));

Route::set('troubleshooting', 'troubleshooting')
	->defaults(array(
		'controller' => 'page',
		'action'     => 'troubleshooting',
	));

Route::set('rest-api', 'api/<action>')
    ->filter(function($route, $params, $request)
    {
    	if ($request->method() !== HTTP_Request::GET)
        {
            return FALSE; // This route only matches GET requests
        }
        // Prefix the method to the action name
        $params['action'] = strtolower($request->method()).'_'.$params['action'];
        return $params; // Returning an array will replace the parameters
    })
    ->defaults(array(
        'controller' => 'api',
    ));

Route::set('default', '(<controller>(/<action>(/<id>)))')
	->defaults(array(
		'controller' => 'home',
		'action'     => 'index',
	));
