<?php defined('SYSPATH') or die('No direct script access.');

class Controller_Admin_Category extends Controller_Template {

	public $template = 'template/admin2_tpl';

	public function before()
	{
		parent::before();

		$this->session = Session::instance();

		//checking ifAdministrator is logged in
		if( ! $this->session->get('user_id'))
		{
			throw HTTP_Exception::factory(404, 'URL not found!');
		}

		$checklogin = MyHelper::check_user_login($this->session->get('user_id'));
		if ($checklogin == 'Y')
		{
			$this->redirect('admin/logout/push');
		}

		$user_role_id = $this->session->get('user_role_id');
		// Only Data Admin can access this module (id 3)
		if ($user_role_id != 3)
		{
			throw HTTP_Exception::factory(500, 'Access not allowed.');
		}
	}

	public function action_index()
	{
		$nav = 'nav_categ';
		if ($this->session->get('nav_link') != $nav)
		{
			$this->session->set('nav_link', $nav);
		}

		$flash_data = $this->session->get_once('flash_data');

		$view = View::factory('admin/category/index');
		$view->navlink = $this->navlink();

		$sql = ORM::factory('ContentType')
			->find_all();

		$data = array();
		$view->main = View::factory('admin/category/form');
		$view->main->data = $data;
		$view->main->sql = $sql;
		$view->main->flash_data = $flash_data;

		$this->template->content = $view;
	}

	/**
	 * List All Created Categories
	 * @return [type] [description]
	 */
	private function navlink()
	{
		$sql = ORM::factory('Category')
			->order_by('name')
			->order_by('id')
			->find_all();

		$link = View::factory('admin/category/navlink');
		$link->header = 'Categories';
		$link->sql = $sql;

		return $link;
	}

	/**
	 * Display list of available templates for this category in ColorBox
	 * @return [type] [description]
	 */
	public function action_select_tpl()
	{
		$this->auto_render = FALSE;

		if (HTTP_Request::POST == $this->request->method() AND $this->request->is_ajax())
		{
			$categ_id = Arr::get($this->request->post(), 'categ_id');
			$ctype_id = Arr::get($this->request->post(), 'ctype_id');
			$templates = Arr::get($this->request->post(), 'tpls');
			$def_tpls = Arr::get($this->request->post(), 'def_tpls');
			$sql = ORM::factory('Template');

			if ($def_tpls)
			{
				$sql->where('id', 'IN', DB::expr('('.$def_tpls.')'));
					// ->where('content_type_id', '=', $ctype_id);
			}

			$tpls = $sql->find_all();

			$view = View::factory('admin/category/tpl_list');
			$view->tpls = $tpls;
			$view->ctype_id = $ctype_id;
			$view->arr_tpls = $templates ? explode(",", $templates) : array();
			// $view->btn_label = $categ_id ? 'Save' : 'Add';
			$view->btn_label = 'Assign';

			$this->response->body($view);
		}
	}

	/**
	 * Check if Category is already existing
	 */
	public function action_check()
	{
		$this->auto_render = FALSE;

		$result = 0;
		if (HTTP_Request::POST == $this->request->method() AND $this->request->is_ajax())
		{
			$name = Arr::get($this->request->post(), 'name');

			$sql = ORM::factory('Category')
				->where('name', '=', $name)
				->find_all();
			$result = $sql->count();
		}

		if ($result)
		{
			echo json_encode(false);
		}
		else
		{
			echo json_encode(true);
		}
	}

	/**
	 * Save new Category or Update an existing one
	 * @return [type] [description]
	 */
	public function action_save()
	{
		$this->auto_render = FALSE;

		if (HTTP_Request::POST == $this->request->method())
		{
			$token = Arr::get($this->request->post(), 'token');

			if (Security::check($token))
			{
				$id = Arr::get($this->request->post(), 'id');
				$name = HTML::chars(Arr::get($this->request->post(), 'name'));
				// $folder_name = URL::title($name, '_');
				$roles = Arr::get($this->request->post(), 'roles');
				$arr_ctype = Arr::get($this->request->post(), 'ctype');
				$ctype = implode(",", $arr_ctype);

				foreach ($arr_ctype as $k=>$d)
				{
					$tpls = Arr::get($this->request->post(), 'tpl_'.$d);
					$tmp_ctype[$d] = $tpls;
				}

				$templates = json_encode($tmp_ctype);

				$content_types = MyHelper::content_types();

				if ($id)
				{
					$sql = ORM::factory('Category', $id);
					$qry = clone $sql;
					$name = $sql->name;
					$action = "Edit";

					// get old data/value
					$arr_tpls = $qry->templates ? json_decode($qry->templates, true) : array();
					$arr_ctypes = MyHelper::get_content_type($qry->content_types);
					$other_info = "Old Value\r\nRole: ".($qry->roles == '4,5,7' ? "Maker, Checker, Publisher" : "Maker, Checker 1, Checker 2, Publisher")."\r\nContent Type(s)\r\n";
					/*
					foreach ($arr_ctypes as $ctype_id => $ctype_name)
             	{
             		$other_info .= "Name: ".$ctype_name."; No. of Templates: ".count(explode(",", Arr::get($arr_tpls, $ctype_id)))."\r\n";
             	}
             	*/
             	foreach ($content_types as $cid => $cname)
             	{
             		$other_info .= "Name: ".$cname."; No. of Templates: ".count(explode(",", Arr::get($arr_tpls, $cid))).(Arr::get($arr_ctypes, $cid) ? ' ON' : ' OFF')."\r\n";
             	}
				}
				else
				{
					$sql = ORM::factory('Category');
					$sql->name = $name;
					$action = "Add";

					$other_info = '';
				}

				$sql->roles = $roles;
				$sql->content_types = $ctype;
				$sql->templates = $templates;
				$sql->save();
				$categ_id = $sql->id;

				// For Activity
				$log_action = "Category";

				$arr_tpls = $templates ? json_decode($templates, true) : array();
				$arr_ctypes = MyHelper::get_content_type($ctype);
				$other_info .= ($other_info ? "\r\n" : "")."New Value\r\nRole: ".($roles == '4,5,7' ? "Maker, Checker, Publisher" : "Maker, Checker 1, Checker 2, Publisher")."\r\nContent Type(s)\r\n";
				/*
				foreach ($arr_ctypes as $ctype_id => $ctype_name)
          	{
          		$other_info .= "Name: ".$ctype_name."; No. of Templates: ".count(explode(",", Arr::get($arr_tpls, $ctype_id)))."\r\n";
          	}
          	*/
          	foreach ($content_types as $cid => $cname)
          	{
          		$other_info .= "Name: ".$cname."; No. of Templates: ".count(explode(",", Arr::get($arr_tpls, $cid))).';'.(Arr::get($arr_ctypes, $cid) ? ' ON' : ' OFF')."\r\n";
          	}

          	$checker_name = $this->session->get('checker_name');
				$log_description = "{$action} \"{$name}\" Category\r\nChecked By: ".$checker_name."\r\n\r\n".$other_info;

				// Log Activity
				Activity::add($this->session->get('user_id'), $log_action, $log_description);

				$data = array(
					'action' => strtolower($action),
					'id' => $categ_id
				);

				$this->session->set('flash_data', json_encode($data));

				$this->redirect('admin/category/msg_view');
			}

			throw HTTP_Exception::factory(500, 'Access not allowed.');
		}

		throw HTTP_Exception::factory(404, 'URL not found!');
	}

	/**
	 * Display Edit Form for Category
	 * @return [type] [description]
	 */
	public function action_edit()
	{
		$id = $this->request->param('id');

		$sql = ORM::factory('Category', $id);
		$data = array(
			'id' => $sql->id,
			'name' => $sql->name,
			'roles' => $sql->roles,
			'content_types' => $sql->content_types,
			'templates' => $sql->templates
		);

		$view = View::factory('admin/category/index');
		$view->navlink = $this->navlink();

		$sql = ORM::factory('ContentType')
			->find_all();

		$view->main = View::factory('admin/category/form');
		$view->main->data = $data;
		$view->main->sql = $sql;
		$view->main->flash_data = '';

		$this->template->content = $view;
	}

	/**
	 * Display Message
	 * @return [type] [description]
	 */
	public function action_msg_view()
	{
		$view = View::factory('admin/category/index');
		$view->navlink = $this->navlink();

		$view->main = View::factory('admin/category/msg_view');

		$flash_data = $this->session->get_once('flash_data');
		$arr_data = json_decode($flash_data, true);

		if (Arr::get($arr_data, 'action') != 'delete')
		{
			$sql = ORM::factory('Category', Arr::get($arr_data, 'id'));
			$view->main->sql = $sql;
		}

		$view->main->arr_data = $arr_data;
		$this->template->content = $view;
	}

	public function action_check_delete()
	{
		$this->auto_render = FALSE;

		if (HTTP_Request::POST == $this->request->method() AND $this->request->is_ajax())
		{
			$token = Arr::get($this->request->post(), 'token');

			if (Security::check($token))
			{
				$category_id = Arr::get($this->request->post(), 'categ_id');

				$user_conflict = FALSE;
				$cms_conflict = FALSE;

				// check if there are user attached to this category
				$sql = ORM::factory('Admin')
					->where(DB::expr('\',\'+').'category_ids'.DB::expr('+\',\''), 'LIKE', "%,$category_id,%")
					->find_all();
				if ($sql->count())
				{
					$user_conflict = TRUE;
				}

				// check if there are pages/cms created using this category
				$sql = ORM::factory('Page')
					->where('category_id', '=', $category_id)
					->find_all();

				if ($sql->count())
				{
					$cms_conflict = TRUE;
				}

				echo json_encode(array('user'=>$user_conflict, 'cms'=>$cms_conflict));
			}
		}
	}

	public function action_delete()
	{
		$this->auto_render = FALSE;

		if (HTTP_Request::POST == $this->request->method())
		{
			$token = Arr::get($this->request->post(), 'token');

			if (Security::check($token))
			{
				require_once Kohana::find_file('vendor', 'Html2Text/Html2Text');
				$comment = Arr::get($this->request->post(), 'comment');
				$html2text = new \Html2Text\Html2Text($comment);
				$comment = $html2text->get_text();
				$category_id = Arr::get($this->request->post(), 'id');

				$sql = ORM::factory('Category', $category_id);

				if ($sql->loaded())
				{
					$name = $sql->name;
					$sql->delete();

					// For Activity
					$log_action = "Category";
					$checker_name = $this->session->get('checker_name');
					$log_description = "Delete \"{$name}\" Category\r\nChecked By: ".$checker_name."\r\n\r\n{$comment}";

					// Log Activity
					Activity::add($this->session->get('user_id'), $log_action, $log_description);

					$this->session->set('flash_data', 'Delete "'.$name.'" category is successful.');

					$this->redirect('admin/category');
				}
			}
		}
	}
}