<?php defined('SYSPATH') or die('No direct script access.');

class Controller_Admin_Links extends Controller_Template {

	public $template = 'template/admin2_tpl';

	public function before()
	{
		parent::before();

		$this->session = Session::instance();

		//checking ifAdministrator is logged in
		if( ! $this->session->get('user_id'))
		{
			throw HTTP_Exception::factory(404, 'URL not found!');
		}

		$checklogin = MyHelper::check_user_login($this->session->get('user_id'));
		if ($checklogin == 'Y')
		{
			$this->redirect('admin/logout/push');
		}

		$user_role_id = $this->session->get('user_role_id');
		// Only Data Admin can access this module (id 3)
		if ($user_role_id != 3)
		{
			throw HTTP_Exception::factory(500, 'Access not allowed.');
		}

		$this->headings = array(
			'related_sites' => 'Related Sites',
			'about_us' => 'About Us',
			'apply_now' => 'Apply Now',
			'promos' => 'Promos',
			'rates' => 'Rates',
			'locator' => 'Locator',
			'contact_us' => 'Contact Us',
			'product_services' => 'Product & Services',
			'market_watcher' => 'Market Watcher',
			'careers' => 'Careers',
			'security' => 'Security',
			'preferred_banking' => 'Preferred Banking',
			'private_banking' => 'Private Banking'
		);

		$nav = 'nav_toplinks';
		if ($this->session->get('nav_link') != $nav)
		{
			$this->session->set('nav_link', $nav);
		}
	}

	public function action_page()
	{
		$label = $this->request->param('id');
		$heading = Arr::get($this->headings, $label);

		$sql = ORM::factory('TopLink')
			->where('label', '=', $label)
			->find_all();

		$flash_data = $this->session->get_once('flash_data');

		$this->template->content = View::factory('admin/top_links/form');
		$this->template->content->heading = $heading;
		$this->template->content->label = $label;
		$this->template->content->sql = $sql;
		$this->template->content->flash_data = $flash_data;
	}

	public function action_save()
	{
		$this->auto_render = FALSE;

		if (HTTP_Request::POST == $this->request->method())
		{
			$token = Arr::get($this->request->post(), 'token');

			if (Security::check($token))
			{
				$sql = ORM::factory('TopLink');
				$label = Arr::get($this->request->post(), 'label');
				$link_type = Arr::get($this->request->post(), 'link_type');
				$sql->label = $label;
				$sql->type = $link_type;
				if ($link_type == 'link')
				{
					$sql->page_link = HTML::chars(Arr::get($this->request->post(), 'page_link'));
					$log_info = "Page Link: ".Arr::get($this->request->post(), 'page_link');
				}
				else
				{
					$sql->menu_label = HTML::chars(Arr::get($this->request->post(), 'menu_label'));
					$sql->menu_link = HTML::chars(Arr::get($this->request->post(), 'menu_link'));
					$log_info = "Menu Label: ".Arr::get($this->request->post(), 'menu_label');
					$log_info .= "Menu Link: ".Arr::get($this->request->post(), 'menu_link');
				}
				$sql->save();

				$this->session->set('flash_data', 'The changes have been saved.');

				// For Activity
				$log_action = "Top/Bottom Links";
				$label_name = Arr::get(MyHelper::top_bottom_links(), $label);
				$checker_name = $this->session->get('checker_name');
				$log_description = "Add \"".$label_name."\"\r\nChecked By: ".$checker_name."\r\n\r\n".$log_info;
				// Log Activity
				Activity::add($this->session->get('user_id'), $log_action, $log_description);

				$this->redirect('admin/top_links/page/'.$label);
			}
		}
	}

	public function action_edit()
	{
		$this->auto_render = FALSE;

		if (HTTP_Request::POST == $this->request->method() AND $this->request->is_ajax())
		{
			$id = Arr::get($this->request->post(), 'id');

			$sql = ORM::factory('TopLink', $id);

			$view = View::factory('admin/top_links/edit_form');
			$view->sql = $sql;

			$this->response->body($view);
		}
	}

	public function action_update()
	{
		$this->auto_render = FALSE;

		if (HTTP_Request::POST == $this->request->method())
		{
			$token = Arr::get($this->request->post(), 'token');

			if (Security::check($token))
			{
				$id = Arr::get($this->request->post(), 'id');
				$label = Arr::get($this->request->post(), 'label');
				$link_type = Arr::get($this->request->post(), 'link_type');

				$sql = ORM::factory('TopLink', $id);
				$c_page_link = html_entity_decode($sql->page_link);
				$c_menu_label = html_entity_decode($sql->menu_label);
				$c_menu_link = html_entity_decode($sql->menu_link);
				$c_type = $sql->type;
				$log_info = "Old Value";
				if ($c_type == 'link')
				{
					$log_info .= "\r\nPage Link: ".$c_page_link;
				}
				else
				{
					$log_info .= "\r\nMenu Label: ".$c_menu_label;
					$log_info .= "\r\nMenu Link: ".$c_menu_link;
				}

				$log_info .= "\r\n\r\nNew Value";
				if ($link_type == 'link')
				{
					$sql->page_link = HTML::chars(Arr::get($this->request->post(), 'page_link'));
					$log_info .= "Page Link: ".Arr::get($this->request->post(), 'page_link');
				}
				else
				{
					$sql->menu_label = HTML::chars(Arr::get($this->request->post(), 'menu_label'));
					$sql->menu_link = HTML::chars(Arr::get($this->request->post(), 'menu_link'));
					$log_info .= "Menu Label: ".Arr::get($this->request->post(), 'menu_label');
					$log_info .= "Menu Link: ".Arr::get($this->request->post(), 'menu_link');
				}
				$sql->save();

				// For Activity
				$log_action = "Top/Bottom Links";
				$label_name = Arr::get(MyHelper::top_bottom_links(), $label);
				$checker_name = $this->session->get('checker_name');
				$log_description = "Update \"".$label_name."\"\r\nChecked By: ".$checker_name."\r\n\r\n".$log_info;
				// Log Activity
				Activity::add($this->session->get('user_id'), $log_action, $log_description);

				$this->redirect('admin/top_links/page/'.$label);
			}
		}
	}

	public function action_delete()
	{
		$this->auto_render = FALSE;

		if (HTTP_Request::POST == $this->request->method())
		{
			$token = Arr::get($this->request->post(), 'token');

			if (Security::check($token))
			{
				$id = Arr::get($this->request->post(), 'id');
				$label = Arr::get($this->request->post(), 'label');
				require_once Kohana::find_file('vendor', 'Html2Text/Html2Text');
				$comment = Arr::get($this->request->post(), 'comment');
				$html2text = new \Html2Text\Html2Text($comment);
				$comment = $html2text->get_text();

				$sql = ORM::factory('TopLink', $id);
				if ($sql->loaded())
				{
					$sql->delete();

					$toplink = Arr::get($this->headings, $label);
					// For Activity
					$checker_name = $this->session->get('checker_name');
					$log_action = "Top/Bottom Links";
					$log_description = "Delete \"".$toplink."\" drop down menu\r\nChecked By: ".$checker_name."\r\n\r\n$comment";

					// Log Activity
					Activity::add($this->session->get('user_id'), $log_action, $log_description);

					$this->session->set('flash_data', 'Delete Top Links ('.$toplink.') drop down menu is successful.');

					$this->redirect('admin/top_links/page/'.$label);
				}
			}
		}
	}

	public function action_cms_list()
	{
		$this->auto_render = FALSE;

		if (HTTP_Request::POST == $this->request->method() AND $this->request->is_ajax())
		{
			$target_id = Arr::get($this->request->post(), 'target_id');

			$current_date = date('Y-m-d', time());

			$sql = ORM::factory('Page')
				->where_open()
				->where('content_type_id', '=', 1)
				->or_where('content_type_id', '>', 6)
				->where_close()
				->where('start_date', '<=', $current_date)
				->where('end_date', '>=', $current_date)
				->where_open()
				->where('status', '=', 'published')
				->or_where('status', '=', 'request-archive')
				->or_where('status', '=', 'pending-archive')
				->or_where('status', '=', 'check-archive')
				->or_where('status', '=', 'rework-archive')
				->where_close()
				->find_all();

			$view = View::factory('admin/cms_list');
			$view->query = $sql;
			$view->target_id = $target_id;
			$view->cms = 0;

			$this->response->body($view);
		}
	}
}