<?php defined('SYSPATH') or die('No direct script access.');

class Controller_Admin_Logs extends Controller_Template {

	public $template = 'template/admin_tpl';

	public function before()
	{
		parent::before();

		$this->session = Session::instance();

		//checking ifAdministrator is logged in
		if( ! $this->session->get('user_id'))
		{
			throw HTTP_Exception::factory(404, 'URL not found!');
		}

		$checklogin = MyHelper::check_user_login($this->session->get('user_id'));
		if ($checklogin == 'Y')
		{
			$this->redirect('admin/logout/push');
		}

		$user_role_id = $this->session->get('user_role_id');
		// Only Level 1 and User Admin can access this module (id 1 and 2)
		if ($user_role_id >= 3)
		{
			throw HTTP_Exception::factory(500, 'Access not allowed.');
		}

		$this->template->navmenu = View::factory('nav/main');
	}

	public function action_index()
	{
		$sub = 'sub_user_logs';
		if ($this->session->get('sub_link') != $sub)
		{
			$this->session->set('sub_link', $sub);
		}

		$id = $this->request->param('id');

		$categ = ORM::factory('Category')
			->find_all();
		$arr_categories = array();
		$arr_categories[] = 'All';
		foreach ($categ as $res)
		{
			$arr_categories[$res->id] = $res->name;
		}

		$flash_data = $this->session->get_once('flash_data');

		$this->template->content = View::factory('admin/logs/form');
		$this->template->content->arr_categories = $arr_categories;
		$this->template->content->user_id = $id;
		$this->template->content->flash_data = $flash_data;
	}

	public function action_get_users()
	{
		$this->auto_render = FALSE;

		if (HTTP_Request::POST == $this->request->method() AND $this->request->is_ajax())
		{
			$user_role_id = $this->session->get('user_role_id');

			$sql = ORM::factory('Admin');

			if ($this->session->get('user_role_id') == 1)
			{
				$sql->where('role_id', '<=', 3)
					->or_where('role_id', '>', 7);
			}
			else if ($this->session->get('user_role_id') == 2)
			{
				$sql->where('role_id', '>=', 4)
					->where('role_id', '<=', 7);
			}

			$qry = $sql->find_all();
			$arr_users = array();
			// $arr_users[] = 'Please select';
			foreach ($qry as $res)
			{
				$arr_users[$res->id] = $res->name;
			}

			echo json_encode($arr_users);
		}
	}

	/**
	 * Get user logs filtered by the criteria and generate CSV file
	 * @return [type] [description]
	 */
	public function action_download()
	{
		$this->auto_render = FALSE;

		if (HTTP_Request::POST == $this->request->method())
		{
			$startdate = date('Y-m-d H:i:s', strtotime(Arr::get($this->request->post(), 'startdate')));
			$enddate = date('Y-m-d 23:59:59', strtotime(Arr::get($this->request->post(), 'enddate')));
			$users = Arr::get($this->request->post(), 'users');
			$category_id = Arr::get($this->request->post(), 'category');
			$user_id = Arr::get($this->request->post(), 'user');
			$control_no = Arr::get($this->request->post(), 'control_no');

			$f1 = date('mdy', time());

			$sql = ORM::factory('Activity')
				->where('date_added', '>=', $startdate)
				->where('date_added', '<=', $enddate);

			$log_info = '';
			if ($users == 'single')
			{
				$sql->where('user_id', '=', $user_id);

				$qry = DB::select('username')
					->from('users')
					->where('id', '=', $user_id)
					->as_object()
					->execute();
				$u = $qry->current();

				$f2 = preg_replace('/\s+/', '', trim($u->username));

				$log_info .= "Users: Single Users (".$u->username.")";
			} else {
				$f2 = 'ALLUSERS';
				$user_ids = '';
				if ($this->session->get('user_role_id') == 1)
				{
					$usql = ORM::factory('Admin')
						->where('role_id', '<=', 3)
						->or_where('role_id', '>', 7)
						->find_all();
				}
				else if ($this->session->get('user_role_id') == 2)
				{
					$usql = ORM::factory('Admin')
						->where('role_id', '>=', 4)
						->where('role_id', '<=', 7)
						->find_all();
				}
				foreach ($usql as $u)
				{
					$user_ids .= ($user_ids ? ',' : '').$u->id;
				}

				$sql->where('user_id', 'IN', DB::expr('('.$user_ids.')'));

				$log_info .= "Users: All Users";
			}

			if ($category_id > 0)
			{
				$sql->where('category_id', '=', $category_id);

				$qry = DB::select('name')
					->from('categories')
					->where('id', '=', $category_id)
					->as_object()
					->execute();
				$c = $qry->current();

				$f2 = preg_replace('/\s+/', '', trim($c->name));
				$f2 = str_replace("/", "-", $f2);

				$log_info .= ($log_info ? "\r\n" : "")."Category: ".$c->name;
			}

			if ($control_no)
			{
				$sql->where('control_no', '=', $control_no);

				$f2 = str_replace('-', '', $control_no);

				$log_info .= ($log_info ? "\r\n" : "")."Control No: ".$control_no;
			}

			$directory = DOCROOT.'media/logs';
			$filename = $f1.'-'.$f2.'-CMS-USERLOGS.csv';

			$logs = $sql->find_all();

			if ($logs->count())
			{
				$handle = fopen($directory.'/'.$filename, 'w+');
				fputcsv($handle, array('Date & Time','User', 'Username', 'Category', 'Role', 'Email', 'Activity', 'Description', 'Control No'));

				foreach ($logs as $res)
				{
					$name = $res->user_name;
					$user_type = $res->user_type;
					$username = $res->user->username;
					$category = $res->category->name ? $res->category->name : 'NA';
					$user_email = $res->user->email;
					$action = $res->action;
					$control_no = $res->control_no;
					$date_added = date('mdy-H:i', strtotime($res->date_added));
					$description = $res->description ? html_entity_decode($res->description) : '';

					fputcsv($handle, array($date_added, $name, $username, $category, $user_type, $user_email, $action, $description, $control_no));
				}

				fclose($handle);

				$log_action = "User Logs";
				$other_info = "Start Date: ".date('Y-m-d', strtotime(Arr::get($this->request->post(), 'startdate')));
				$other_info .= "\r\nEnd Date: ".date('Y-m-d', strtotime(Arr::get($this->request->post(), 'enddate')));
				$other_info .= "\r\n".$log_info."\r\nFilename: ".$filename;

				$checker_name = $this->session->get('checker_name');
				$log_description = "Downloaded User Logs\r\nChecked By: ".$checker_name."\r\n\r\n".$other_info;

				// Log Activity
				Activity::add($this->session->get('user_id'), $log_action, $log_description);

				$this->response->send_file($directory.'/'.$filename);
			}
			else
			{
				$this->session->set('flash_data', 'There are no logs matching the criteria specified.  Please try again.');
				$this->redirect('admin/logs');
			}
		}
	}

	public function action_get_control_no()
	{
		$this->auto_render = FALSE;

		$params = $this->request->query();
		$control_no = Arr::get($params, 'q');

		$sql = DB::select('control_no')
			->from('activities')
			->where('control_no', 'LIKE', '%'.$control_no.'%')
			->group_by('control_no')
			->as_object()
			->execute();

		$arr_c = array();
		foreach ($sql as $res)
		{
			$arr_c[$res->control_no] = $res->control_no;
		}

		echo json_encode($arr_c);
	}
}