<?php defined('SYSPATH') or die('No direct script access.');

class Controller_Admin_Media extends Controller_Template {

	public $template = 'template/admin2_tpl';

	public function before()
	{
		parent::before();

		$this->session = Session::instance();

		//checking ifAdministrator is logged in
		if( ! $this->session->get('user_id'))
		{
			throw HTTP_Exception::factory(404, 'URL not found!');
		}

		$checklogin = MyHelper::check_user_login($this->session->get('user_id'));
		if ($checklogin == 'Y')
		{
			$this->redirect('admin/logout/push');
		}

		$user_role_id = $this->session->get('user_role_id');
		// Only Data Admin can access this module (id 3)
		if ($user_role_id != 3)
		{
			throw HTTP_Exception::factory(500, 'Access not allowed.');
		}
	}

	public function action_index()
	{
		$nav = 'nav_media';
		if ($this->session->get('nav_link') != $nav)
		{
			$this->session->set('nav_link', $nav);
		}

		$view = View::factory('admin/media/index');
		$view->navlink = $this->navlink();
		$view->main = '';

		$this->template->content = $view;
	}

	private function navlink()
	{
		$sql = ORM::factory('Category')
			->order_by('name')
			->order_by('id')
			->find_all();

		$link = View::factory('admin/media/navlink');
		$link->header = 'Category and Media Folder';
		$link->sql = $sql;

		return $link;
	}

	public function action_list()
	{
		$category_id = $this->request->param('id');

		$csql = ORM::factory('Category', $category_id);
		$category_name = $csql->name;

		$view = View::factory('admin/media/index');
		$view->navlink = $this->navlink();

		// get the list of pages created under this category
		$pages = ORM::factory('Page')
			->where('category_id', '=', $category_id)
			->find_all();

		if ($pages->count())
		{
			$str_pages = '';
			foreach ($pages as $p)
			{
				$str_pages .= ($str_pages ? ',' : '').$p->id;
			}

			$sql = ORM::factory('Media')
				->where('page_id', 'IN', DB::expr('('.$str_pages.')'))
				->order_by('page_id')
				->find_all();

			$view->main = View::factory('admin/media/file_list');
			$view->main->header = $category_name;
			$view->main->sql = $sql;

		} else {
			$view->main = '<fieldset><legend>'.$category_name.'</legend><h4>No Files</h4></fieldset>';
		}

		$this->template->content = $view;
	}

	public function action_remove_files()
	{
		$this->auto_render = FALSE;

		if (HTTP_Request::POST == $this->request->method())
		{
			$token = Arr::get($this->request->post(), 'token');

			if (Security::check($token))
			{
				$files = Arr::get($this->request->post(), 'file');
				$category_name = Arr::get($this->request->post(), 'category_name');

				if (is_array($files))
				{
					$docroot = DOCROOT.'media/uploads';
					foreach ($files as $file_id => $file_name)
					{
						if (is_file($docroot.'/'.$file_name) AND file_exists($docroot.'/'.$file_name))
						{
							@unlink($docroot.'/'.$file_name);

							DB::delete('media')
								->where('id', '=', $file_id)
								->execute();
						}
					}

					// For Activity
					$log_action = "Media Files";
					$checker_name = $this->session->get('checker_name');
					$log_description = "Deleted media files for Category \"".$category_name."\"\r\nChecked By: ".$checker_name;

					// Log Activity
					Activity::add($this->session->get('user_id'), $log_action, $log_description);
				}

				$this->redirect('admin/media/msg_view');

				exit;
			}
		}
	}

	/**
	 * Display Message
	 * @return [type] [description]
	 */
	public function action_msg_view()
	{
		$view = View::factory('admin/media/index');
		$view->navlink = $this->navlink();

		$view->main = View::factory('admin/media/msg_view');

		$flash_data = $this->session->get_once('flash_data');
		$arr_data = json_decode($flash_data, true);

		if (Arr::get($arr_data, 'action') != 'delete')
		{
			$sql = ORM::factory('ContentType', Arr::get($arr_data, 'id'));
			$view->main->sql = $sql;
		}

		$view->main->arr_data = $arr_data;
		$this->template->content = $view;
	}
}