<?php defined('SYSPATH') or die('No direct script access.');

class Controller_Admin_Navtree extends Controller_Template {

	public $template = 'template/admin2_tpl';

	public function before()
	{
		parent::before();

		$this->session = Session::instance();

		//checking ifAdministrator is logged in
		if( ! $this->session->get('user_id'))
		{
			throw HTTP_Exception::factory(404, 'URL not found!');
		}

		$checklogin = MyHelper::check_user_login($this->session->get('user_id'));
		if ($checklogin == 'Y')
		{
			$this->redirect('admin/logout/push');
		}

		$user_role_id = $this->session->get('user_role_id');
		// Only Data Admin can access this module (id 3)
		if ($user_role_id != 3)
		{
			throw HTTP_Exception::factory(500, 'Access not allowed.');
		}
	}

	/**
	 * View Navigation Tree menu main page
	 * @return [type] [description]
	 */
	public function action_index()
	{
		$nav = 'nav_tree';
		if ($this->session->get('nav_link') != $nav)
		{
			$this->session->set('nav_link', $nav);
		}

		$view = View::factory('admin/navtree/index');

		$sql = ORM::factory('Category')
			->order_by('name')
			->order_by('id')
			->find_all();
		$arr_category = array();
		$arr_category[] = '';
		foreach ($sql as $res)
		{
			$arr_category[$res->id] = $res->name;
		}

		$view->main = View::factory('admin/navtree/list');
		$view->main->arr_category = $arr_category;

		$this->template->content = $view;
	}
	/**
	 * Manage Navigation Tree menu main page
	 * @return [type] [description]
	 */
	public function action_add()
	{
		$view = View::factory('admin/navtree/index');

		// Get the Category
		$sql = ORM::factory('Category')
			->order_by('name')
			->order_by('id')
			->find_all();
		$arr_category = array();
		$arr_category[] = '';
		foreach ($sql as $res)
		{
			$arr_category[$res->id] = $res->name;
		}

		$view->main = View::factory('admin/navtree/form');
		$view->main->arr_category = $arr_category;

		$this->template->content = $view;
	}

	/**
	 * Display the add sub category form in Manage Navigation Tree
	 * @return [type] [description]
	 */
	public function action_get_category()
	{
		$this->auto_render = FALSE;

		if (HTTP_Request::POST == $this->request->method() AND $this->request->is_ajax())
		{
			$categ_id = Arr::get($this->request->post(), 'categ_id');
			$level = Arr::get($this->request->post(), 'level');
			$parent_id = Arr::get($this->request->post(), 'parent_id');
			$category_id = Arr::get($this->request->post(), 'category_id');

			switch ($level)
			{
				case 'sub1':
					$data_for = 'sub2';
					$lvl = 1;
					$header = 'Sub-Category 1';
					$category_id = $categ_id;
					break;
				case 'sub2':
					$data_for = 'sub3';
					$lvl = 2;
					$header = 'Sub-Category 2';
					break;
				case 'sub3':
					$data_for = '';
					$lvl = 3;
					$header = 'Sub-Category 3';
			}

			$sql = ORM::factory('Navtree')
				->where('category_id', '=', $category_id)
				->where('level', '=', $lvl)
				->where('parent_id', '=', $parent_id)
				->order_by('name')
				->find_all();

			$arr_category = array();
			$arr_category[] = '';
			foreach ($sql as $res)
			{
				$arr_category[$res->id] = $res->name;
			}

			// generate an html view for the select box
			$view = View::factory('admin/navtree/category_select');
			$view->arr_category = $arr_category;

			// check if we can delete this category
			// if no subcategory and if there are no active page attached to it.
			$categ_level = $lvl - 1;

			$check_posting = $this->check_posting($categ_id, $categ_level, $category_id);
			// echo 'Count = '.$sql->count().'<br />';
			// echo 'Posting = '.$check_posting.'<br />';
			// echo 'Categ ID = '.$categ_id.'<br />';
			// echo 'Categ Level = '.$categ_level.'<br />';
			if ($sql->count() OR $check_posting)
			{
				// disable delete
				$del_btn = " disabled=\"disabled\"";
			} else {
				$del_btn = "";
			}
			$view->data_for = $data_for;
			$view->level = $lvl;
			$view->header = $header;
			$view->category_id = $category_id;
			$view->parent_id = $parent_id;
			$view->del_btn = $del_btn;
			$view->categ_level = $categ_level;

			$this->response->body($view);
		}
	}

	private function check_posting($category_id, $categ_level, $maincateg_id)
	{
		$current_date = date('Y-m-d', time());

		$sql = ORM::factory('Page')
			->where('start_date', '<=', $current_date)
			->where('end_date', '>=', $current_date)
			->where_open()
			->where('status', '!=', 'archived')
			->and_where('status', '!=', 'draft')
			->and_where('status', '!=', 'denied')
			->and_where('status', '!=', 'rework-archive')
			->and_where('status', '!=', 'check-archive')
			->where_close()
			->where('category_id', '=', $maincateg_id);

		if ($categ_level == 1)
		{
			$sql->where('sub_category1', '=', $category_id)
				->where_open()
				->where('sub_category2', 'IS', NULL)
				->or_where('sub_category2', '=', 0)
				->where_close()
				->where_open()
				->where('sub_category3', 'IS', NULL)
				->or_where('sub_category3', '=', 0)
				->where_close();
		}
		else if ($categ_level == 2)
		{
			$sql->where('sub_category1', '!=', 0)
				->where('sub_category2', '=', $category_id)
				->where_open()
				->where('sub_category3', 'IS', NULL)
				->or_where('sub_category3', '=', 0)
				->where_close();
		}
		else if ($categ_level == 3)
		{
			$sql->where('sub_category1', '!=', 0)
				->where('sub_category2', '!=', 0)
				->where('sub_category3', '=', $category_id);
		}
		$qry = $sql->find_all();

		return $qry->count();
	}

	/**
	 * Display Sub-Category Dropdown for display in View Navigation Tree
	 * @return [type] [description]
	 */
	public function action_get_category_2()
	{
		$this->auto_render = FALSE;

		if (HTTP_Request::POST == $this->request->method() AND $this->request->is_ajax())
		{
			$categ_id = Arr::get($this->request->post(), 'categ_id');
			$level = Arr::get($this->request->post(), 'level');
			$parent_id = Arr::get($this->request->post(), 'parent_id');
			$category_id = Arr::get($this->request->post(), 'category_id');

			$lvl = 0;
			switch ($level)
			{
				case 'sub1':
					$data_for = 'sub2';
					$lvl = 1;
					$header = 'Sub-Category 1';
					$category_id = $categ_id;
					break;
				case 'sub2':
					$data_for = 'sub3';
					$lvl = 2;
					$header = 'Sub-Category 2';
					break;
				case 'sub3':
					$data_for = '';
					$lvl = 3;
					$header = 'Sub-Category 3';
			}

			if ($lvl > 0)
			{
				$sql = ORM::factory('Navtree')
					->where('category_id', '=', $category_id)
					->where('level', '=', $lvl)
					->where('parent_id', '=', $parent_id)
					->find_all();

				$arr_category = array();
				$arr_category[] = '';
				foreach ($sql as $res)
				{
					$arr_category[$res->id] = $res->name;
				}

				$categ_level = $lvl - 1;
				$check_page = $this->check_page($categ_id, $categ_level);
				// echo $check_page.' test ';

				// generate an html view for the select box
				$view = View::factory('admin/navtree/category_select_2');
				$view->arr_category = $arr_category;

				$view->data_for = $data_for;
				$view->level = $lvl;
				$view->header = $header;
				$view->category_id = $category_id;
				$view->parent_id = $parent_id;

				$view = array(
					'htm' => $view->render(),
					'viewpage' => $check_page
				);
			} else {
				$categ_level = 3;
				$check_page = $this->check_page($categ_id, $categ_level);

				$view = array(
					'htm' => NULL,
					'viewpage' => $check_page
				);
			}

			echo json_encode($view);
			// $this->response->body($view);
		}
	}

	/**
	 * Check if sub category is already existing
	 */
	public function action_check()
	{
		$this->auto_render = FALSE;

		$result = 0;
		if (HTTP_Request::POST == $this->request->method() AND $this->request->is_ajax())
		{
			$name = Arr::get($this->request->post(), 'subcateg_name');

			$sql = ORM::factory('Navtree')
				->where('name', '=', $name)
				->find_all();
			$result = $sql->count();

			if ( ! $result)
			{
				// check in category if already existing
				$sql = ORM::factory('Category')
					->where('name', '=', $name)
					->find_all();
				$result = $sql->count();
			}
		}

		if ($result)
		{
			echo json_encode(false);
		}
		else
		{
			echo json_encode(true);
		}
	}

	/**
	 * Saves or Update a Sub Category
	 * @return [type] [description]
	 */
	public function action_save_categ()
	{
		$this->auto_render = FALSE;

		if (HTTP_Request::POST == $this->request->method())
		{
			$token = Arr::get($this->request->post(), 'token');

			if (Security::check($token))
			{
				$category_id = Arr::get($this->request->post(), 'category_id');
				$name = trim(HTML::chars(Arr::get($this->request->post(), 'subcateg_name')));
				$level = Arr::get($this->request->post(), 'level');
				$parent_id = Arr::get($this->request->post(), 'parent_id');

				$sub_id = Arr::get($this->request->post(), 'sub_id');

				$qry = ORM::factory('Category', $category_id);
				$category_name = $qry->name;
				$checker_name = $this->session->get('checker_name');
				if ($sub_id)
				{
					// if this has value, we're in edit mode
					$sql = ORM::factory('Navtree', $sub_id);
					$old_name = $sql->name;
					$action = 'Edit';
					$log_description = "{$action} Sub-Category {$level} change \"{$old_name}\" to \"{$name}\" for \"{$category_name}\" Category";
				}
				else
				{
					$sql = ORM::factory('Navtree');
					$action = 'Add';
					$log_description = "{$action} Sub-Category {$level} \"{$name}\" for \"{$category_name}\" Category";
				}

				$log_description .= "\r\nChecked By: ".$checker_name;

				$sql->category_id = $category_id;
				$sql->name = $name;
				$sql->parent_id = $parent_id;
				$sql->level = $level;
				$sql->save();

				$sub_id = $sql->id;

				// For Activity
				$log_action = "Navigation Tree";

				// Log Activity
				Activity::add($this->session->get('user_id'), $log_action, $log_description);

				$data = array(
					'action' => strtolower($action),
					'id' => $sub_id
				);

				$this->session->set('flash_data', json_encode($data));

				$this->redirect('admin/navtree/msg_view');
			}

			throw HTTP_Exception::factory(500, 'Access not allowed.');
		}

		throw HTTP_Exception::factory(404, 'URL not found!');
	}

	/**
	 * Display Message
	 * @return [type] [description]
	 */
	public function action_msg_view()
	{
		$view = View::factory('admin/navtree/index');

		$view->main = View::factory('admin/navtree/msg_view');

		$flash_data = $this->session->get_once('flash_data');
		$arr_data = json_decode($flash_data, true);

		if (Arr::get($arr_data, 'action') != 'delete')
		{
			$sql = ORM::factory('Navtree', Arr::get($arr_data, 'id'));
			$parent_id = $sql->parent_id;
			$level = $sql->level;
			$categ = $sql->category->name;
			$date_added = $sql->date_added;
			$subcateg = $sql->name;

			$navtree = '';
			if ($level == 1)
			{
				$navtree = $categ.' / '.$subcateg;
			}
			else if ($level == 2)
			{
				$sql = ORM::factory('Navtree', $parent_id);
				$parent_categ = $sql->name;

				$navtree = $categ.' / '.$parent_categ.' / '.$subcateg;
			}
			else if ($level == 3)
			{
				$sql = ORM::factory('Navtree', $parent_id);
				$sub_categ2 = $sql->name;
				$sub_id2 = $sql->parent_id;

				$sql = ORM::factory('Navtree', $sub_id2);
				$sub_categ = $sql->name;

				$navtree = $categ.' / '.$sub_categ.' / '.$sub_categ2.' / '.$subcateg;
			}

			$view->main->navtree = $navtree;
			$view->main->date_added = $date_added;
		}

		$view->main->arr_data = $arr_data;

		$this->template->content = $view;
	}

	/**
	 * Display the Add Sub-Category form field
	 * @return [type] [description]
	 */
	public function action_add_category()
	{
		$this->auto_render = FALSE;

		if (HTTP_Request::POST == $this->request->method() AND $this->request->is_ajax())
		{
			$level = Arr::get($this->request->post(), 'level');

			$htm = '<input type="text" name="subcateg_name" id="subcateg_name" value="" class="span3" maxlength="70">
			<button type="submit" data-level="'.$level.'" class="btn btn-success btn-small btn-save-categ">Save</button>
			<button type="button" data-level="'.$level.'" class="btn btn-small btn-back-categ">Back</button>';

			$this->response->body($htm);
		}
	}

	/**
	 * Display the Update Category Field
	 * @return [type] [description]
	 */
	public function action_edit_category()
	{
		$this->auto_render = FALSE;

		if (HTTP_Request::POST == $this->request->method() AND $this->request->is_ajax())
		{
			$name = Arr::get($this->request->post(), 'name');
			$sub_id = Arr::get($this->request->post(), 'sub_id');
			$level = Arr::get($this->request->post(), 'level');
			$data_for = Arr::get($this->request->post(), 'data_for');

			$htm = '<input type="text" name="subcateg_name" id="subcateg_name" value="'.$name.'" class="span3">
			<input type="hidden" name="orig_name_'.$level.'" id="orig_name_'.$level.'" value="'.$name.'">
			<button type="submit" data-level="'.$level.'" class="btn btn-success btn-small btn-update-categ">Save</button>
			<button type="button" data-level="'.$level.'" data-for="'.$data_for.'" class="btn btn-small btn-back-update-categ">Back</button>
			<input type="hidden" name="sub_id" value="'.$sub_id.'">';

			$this->response->body($htm);
		}
	}

	public function action_generate_table()
	{
		$this->auto_render = FALSE;

		if (HTTP_Request::POST == $this->request->method() AND $this->request->is_ajax())
		{
			$category = Arr::get($this->request->post(), 'category');
			$subcategory_1 = Arr::get($this->request->post(), 'subcategory_1');
			$subcategory_2 = Arr::get($this->request->post(), 'subcategory_2');
			$category_name = Arr::get($this->request->post(), 'category_name');
			$filter = Arr::get($this->request->post(), 'filter');

			// $sql = ORM::factory('Navtree')

			$include = true;
			$sql = DB::select()
				->from('navtree')
				->where('category_id', '=', $category);
			if ($subcategory_1 AND $subcategory_2) {
				$sql->where_open()
					->where('id', '=', $subcategory_1)
					->or_where('id', '=', $subcategory_2)
					->where_close()
					->or_where_open()
					->where('parent_id', '=', $subcategory_2)
					->where('level', '=', 3)
					->where_close();
				$include = false;
			} else if ($subcategory_1 AND ! $subcategory_2) {
				$sql->where('id', '=', $subcategory_1)
					->or_where_open()
					->where('parent_id', '=', $subcategory_1)
					->where('level', '=', 2)
					->where_close();
			}

			// $navsql = $sql->order_by('level')->find_all();
			$navsql = $sql->order_by('level')->as_object()->execute();
			// echo Database::instance()->last_query;

			$current_date = date('Y-m-d', time());

			$arr_nav = array();
			$arr_nav2 = array();
			$arr_nav3 = array();

			if ($navsql->count())
			{
				foreach ($navsql as $res)
				{
					$category_id = $res->id;
					$subcategory_name = $res->name;
					$parent_id = $res->parent_id;
					$level = $res->level;

					if ($level == 1) {
						$arr_nav[$category_id] = $subcategory_name;
					} else if ($level == 2) {
						$arr_nav2[$parent_id][$category_id] = $subcategory_name;
					} else {
						$arr_nav3[$parent_id][$category_id] = $subcategory_name;
					}
				}
			}
			/*
			else
			{
				$sql = DB::select('name')
					->from('categories')
					->where('id', '=', $category)
					->as_object()
					->execute();
				$res = $sql->current();
				$arr_nav[$category] = $res->name;
			}
			*/

			/*
			$arr_nav[$category]['name'] = $category_name;
			foreach ($navsql as $res)
			{
				$category_id = $res->id;
				$subcategory_name = $res->name;
				$parent_id = $res->parent_id;
				$level = $res->level;

				if ($level == 1) {

					if ($filter != 0)
					{
						// Active Page
						$p = DB::select('id')
							->from('pages')
							->where('category_id', '=', $category)
							->where_open()
							->where('sub_category1', '=', $category_id)
							->where_close()
							->where_open()
							->where('sub_category2', 'IS', NULL)
							->or_where('sub_category2', '=', 0)
							->where_close()
							->where_open()
							->where('sub_category3', 'IS', NULL)
							->or_where('sub_category3', '=', 0)
							->where_close()
							->where('start_date', '<=', $current_date)
							->where('end_date', '>=', $current_date)
							->where_open()
							->where('content_type_id', '=', 1)
							->or_where('content_type_id', '>', 6)
							->where_close()
							->where_open()
							->where('status', '=', 'published')
							->or_where('status', '=', 'request-archive')
							->or_where('status', '=', 'pending-archive')
							->or_where('status', '=', 'check-archive')
							->where_close();
						$psql = $p->as_object()->execute();
						$c = count($psql);
					}

					if ($filter == 1) // Active Pages
					{
						if ($c > 0) {
							// $arr_nav[$category_id] = $subcategory_name;
							$arr_nav[$category]['sub_1'][$category_id]['name'] = $subcategory_name;
						}
					} else if ($filter == 2) { // Inactive Pages
						if ($c == 0) {
							// $arr_nav[$category_id] = $subcategory_name;
							$arr_nav[$category]['sub_1'][$category_id]['name'] = $subcategory_name;
						}
					} else { // All (Active / Inactive)
						// $arr_nav[$category_id] = $subcategory_name;
						$arr_nav[$category]['sub_1'][$category_id]['name'] = $subcategory_name;
					}

					$arr_par1[$category_id] = $subcategory_name;
				} else if ($level == 2) {
					if ($filter != 0)
					{
						// Active Page
						$p = DB::select('id')
							->from('pages')
							->where('category_id', '=', $category)
							->where('sub_category1', '=', $parent_id)
							->where('sub_category2', '=', $category_id)
							->where_open()
							->where('sub_category3', 'IS', NULL)
							->or_where('sub_category3', '=', 0)
							->where_close()
							->where('start_date', '<=', $current_date)
							->where('end_date', '>=', $current_date)
							->where_open()
							->where('content_type_id', '=', 1)
							->or_where('content_type_id', '>', 6)
							->where_close()
							->where_open()
							->where('status', '=', 'published')
							->or_where('status', '=', 'request-archive')
							->or_where('status', '=', 'pending-archive')
							->or_where('status', '=', 'check-archive')
							->where_close();
						$psql = $p->as_object()->execute();
						$c = count($psql);
					}

					if ($filter == 1)
					{
						if ($c > 0) {
							// $arr_nav2[$parent_id][$category_id] = $subcategory_name;
							$arr_nav[$category]['sub_1'][$parent_id]['sub_2'][$category_id]['name'] = $subcategory_name;
						}
					} else if ($filter == 2) {
						if ($c == 0) {
							// $arr_nav2[$parent_id][$category_id] = $subcategory_name;
							$arr_nav[$category]['sub_1'][$parent_id]['sub_2'][$category_id]['name'] = $subcategory_name;
						}
					} else {
						// $arr_nav2[$parent_id][$category_id] = $subcategory_name;
						$arr_nav[$category]['sub_1'][$parent_id]['sub_2'][$category_id]['name'] = $subcategory_name;
					}
					$arr_par2[$category_id] = $subcategory_name;
				} else {
					if ($filter != 0)
					{
						// Active Page
						$p = DB::select('id')
							->from('pages')
							->where('category_id', '=', $category)
							->where('sub_category1', '!=', 0)
							->where('sub_category2', '=', $parent_id)
							->where('sub_category3', '=', $category_id)
							->where_open()
							->where('sub_category3', 'IS', NULL)
							->or_where('sub_category3', '=', 0)
							->where_close()
							->where('start_date', '<=', $current_date)
							->where('end_date', '>=', $current_date)
							->where_open()
							->where('content_type_id', '=', 1)
							->or_where('content_type_id', '>', 6)
							->where_close()
							->where_open()
							->where('status', '=', 'published')
							->or_where('status', '=', 'request-archive')
							->or_where('status', '=', 'pending-archive')
							->or_where('status', '=', 'check-archive')
							->where_close();
						$psql = $p->as_object()->execute();
						$c = count($psql);
					}

					/*
					if ($filter == 1)
					{
						if ($c > 0) {
							// $arr_nav2[$parent_id][$category_id] = $subcategory_name;
							$arr_nav[$category]['sub_1'][$sub_2_id]['sub_2'][$parent_id]['name']['sub_3'][$category_id] = $subcategory_name;
						}
					} else if ($filter == 2) {
						if ($c == 0) {
							// $arr_nav2[$parent_id][$category_id] = $subcategory_name;
							$arr_nav[$category]['sub_1'][$sub_2_id]['sub_2'][$parent_id]['name']['sub_3'][$category_id] = $subcategory_name;
						}
					} else {
						// $arr_nav2[$parent_id][$category_id] = $subcategory_name;
						$arr_nav[$category]['sub_1'][$sub_2_id]['sub_2'][$parent_id]['name']['sub_3'][$category_id] = $subcategory_name;
					}
					**
					// $arr_nav3[$parent_id][$category_id] = $subcategory_name;
				}
			}
			*/
			// echo "<pre>";
			// var_dump($arr_nav);
			// var_dump($arr_nav2);
			// var_dump($arr_nav3);
			// echo "</pre>";
			// exit;

			$view = View::factory('admin/navtree/navtree_list');
			$view->arr_nav = $arr_nav;
			$view->arr_nav2 = $arr_nav2;
			$view->arr_nav3 = $arr_nav3;
			$view->category_name = $category_name;
			$view->subcateg_1 = $subcategory_1;
			$view->subcateg_2 = $subcategory_2;
			$view->category = $category;
			$view->include = $include;
			$view->filter = $filter;

			$this->response->body($view);
		}
	}

	public function action_download_csv()
	{
		$this->auto_render = FALSE;

		if (HTTP_Request::POST == $this->request->method())
		{
			$category = Arr::get($this->request->post(), 'category');
			$subcategory_1 = Arr::get($this->request->post(), 'subcategory_1');
			$subcategory_2 = Arr::get($this->request->post(), 'subcategory_2');
			$category_name = Arr::get($this->request->post(), 'category_name');
			$filter = Arr::get($this->request->post(), 'filter');

			$include = true;
			$sql = DB::select()
				->from('navtree')
				->where('category_id', '=', $category);
			if ($subcategory_1 AND $subcategory_2) {
				$sql->where_open()
					->where('id', '=', $subcategory_1)
					->or_where('id', '=', $subcategory_2)
					->where_close()
					->or_where_open()
					->where('parent_id', '=', $subcategory_2)
					->where('level', '=', 3)
					->where_close();
				$include = false;
			} else if ($subcategory_1 AND ! $subcategory_2) {
				$sql->where('id', '=', $subcategory_1)
					->or_where_open()
					->where('parent_id', '=', $subcategory_1)
					->where('level', '=', 2)
					->where_close();
			}

			$navsql = $sql->order_by('level')->as_object()->execute();

			$arr_nav = array();
			$arr_nav2 = array();
			$arr_nav3 = array();
			foreach ($navsql as $res)
			{
				$category_id = $res->id;
				$subcategory_name = $res->name;
				$parent_id = $res->parent_id;
				$level = $res->level;

				if ($level == 1) {
					$arr_nav[$category_id] = $subcategory_name;
				} else if ($level == 2) {
					$arr_nav2[$parent_id][$category_id] = $subcategory_name;
				} else {
					$arr_nav3[$parent_id][$category_id] = $subcategory_name;
				}
			}

			$directory = DOCROOT.'media/logs';
			$filename = 'NavigationTree.csv';

			$handle = fopen($directory.'/'.$filename, 'w+');
			fputcsv($handle, array('Category','Sub-Category 1', 'Sub-Category 2', 'Sub-Category 3'));

			if (count($arr_nav) > 0)
			{
				$cid = $category;
				foreach ($arr_nav as $sub_1 => $subcategory_1)
				{
					if ($include) {
						if ($filter != 0)
						{
							$check_nav = MyHelper::check_navstring($cid, $sub_1, NULL, NULL);
							if ($filter == 2)
							{
								$check_nav = ! $check_nav;
							}
						} else {
							$check_nav = 1;
						}

						if ($check_nav)
						{
							fputcsv($handle, array($category_name, $subcategory_1));
						}
					}

					if (array_key_exists($sub_1, $arr_nav2))
					{
						foreach ($arr_nav2[$sub_1] as $sub_2 => $subcategory_2)
						{
							if ($filter != 0)
							{
								$check_nav = MyHelper::check_navstring($cid, $sub_1, $sub_2, NULL);
								if ($filter == 2)
								{
									$check_nav = ! $check_nav;
								}
							} else {
								$check_nav = 1;
							}

							if ($check_nav)
							{
								fputcsv($handle, array($category_name, $subcategory_1, $subcategory_2));
							}

							if (array_key_exists($sub_2, $arr_nav3))
							{
								foreach ($arr_nav3[$sub_2] as $sub_id_3 => $subcategory_3)
								{
									if ($filter != 0)
									{
										$check_nav = MyHelper::check_navstring($cid, $sub_1, $sub_2, $sub_id_3);
										if ($filter == 2)
										{
											$check_nav = ! $check_nav;
										}
									} else {
										$check_nav = 1;
									}

									if ($check_nav)
									{
										fputcsv($handle, array($category_name, $subcategory_1, $subcategory_2, $subcategory_3));
									}
								}
							}
						}
					}
				}
			}
			else
			{
				fputcsv($handle, array($category_name));
			}

			fclose($handle);

			// For Activity
			$log_action = "Navigation Tree";
			$checker_name = $this->session->get('checker_name');
			$log_description = "Download Navigation Tree with file name ".$filename."\r\nChecked By: ".$checker_name;

			// Log Activity
			Activity::add($this->session->get('user_id'), $log_action, $log_description);

			$this->response->send_file($directory.'/'.$filename);
		}
	}

	public function action_delete_category()
	{
		$this->auto_render = FALSE;

		$return = 0;
		if (HTTP_Request::POST == $this->request->method() AND $this->request->is_ajax())
		{
			$categ_id = Arr::get($this->request->post(), 'categ_id');
			$level = Arr::get($this->request->post(), 'level');

			$qry = ORM::factory('Category', $category_id);
			$category_name = $qry->name;

			$sql = ORM::factory('Navtree')
				->where('id', '=', $categ_id)
				->where('level', '=', $level)
				->find();

			if ($sql->id)
			{
				$name = $sql->name;

				$sql->delete();

				// For Activity
				$log_action = 'Navigation Tree';
				$checker_name = $this->session->get('checker_name');
				$log_description = "Delete Sub-Category {$level} \"{$name}\" for \"{$category_name}\" Category\r\nChecked By".$checker_name;

				// Log Activity
				Activity::add($this->session->get('user_id'), $log_action, $log_description);

				$return = 1;
			}
		}

		echo $return;
	}

	private function check_page($category_id, $categ_level)
	{
		$current_date = date('Y-m-d', time());

		$sql = ORM::factory('Page')
			->where('start_date', '<=', $current_date)
			->where('end_date', '>=', $current_date)
			->where_open()
			->where('status', '=', 'published')
			->or_where('status', '=', 'request-archive')
			->or_where('status', '=', 'pending-archive')
			->or_where('status', '=', 'check-archive')
			->where_close()
			->where_open()
			->where('content_type_id', '=', 1)
			->or_where('content_type_id', '>', 7)
			->where_close();

		if ($categ_level == 0)
		{
			// Category Level
			$sql->where('category_id', '=', $category_id)
				->where_open()
				->where('sub_category1', 'IS', NULL)
				->or_where('sub_category1', '=', 0)
				->where_close()
				->where_open()
				->where('sub_category2', 'IS', NULL)
				->or_where('sub_category2', '=', 0)
				->where_close()
				->where_open()
				->where('sub_category3', 'IS', NULL)
				->or_where('sub_category3', '=', 0)
				->where_close();
		}
		else if ($categ_level == 1)
		{
			$sql->where('category_id', '!=', 0)
				->where('sub_category1', '=', $category_id)
				->where_open()
				->where('sub_category2', 'IS', NULL)
				->or_where('sub_category2', '=', 0)
				->where_close()
				->where_open()
				->where('sub_category3', 'IS', NULL)
				->or_where('sub_category3', '=', 0)
				->where_close();
		}
		else if ($categ_level == 2)
		{
			$sql->where('category_id', '!=', 0)
				->where('sub_category1', '!=', 0)
				->where('sub_category2', '=', $category_id)
				->where_open()
				->where('sub_category3', 'IS', NULL)
				->or_where('sub_category3', '=', 0)
				->where_close();
		}
		else if ($categ_level == 3)
		{
			$sql->where('category_id', '!=', 0)
				->where('sub_category1', '!=', 0)
				->where('sub_category2', '!=', 0)
				->where('sub_category3', '=', $category_id);
		}

		$sql->order_by('id', 'DESC');
		$qry = $sql->find();

		return $qry->id;
	}

}