<?php defined('SYSPATH') or die('No direct script access.');

class Controller_Admin_Users extends Controller_Template {

	public $template = 'template/admin_tpl';

	public function before()
	{
		parent::before();

		$this->session = Session::instance();

		//checking ifAdministrator is logged in
		if( ! $this->session->get('user_id'))
		{
			throw HTTP_Exception::factory(404, 'URL not found!');
		}

		$checklogin = MyHelper::check_user_login($this->session->get('user_id'));
		if ($checklogin == 'Y')
		{
			$this->redirect('admin/logout/push');
		}

		$user_role_id = $this->session->get('user_role_id');
		// Only Level 1 and User Admin can access this module (id 1 and 2)
		if ($user_role_id >= 3)
		{
			throw HTTP_Exception::factory(500, 'Access not allowed.');
		}

		$this->template->navmenu = View::factory('nav/main');
	}

	/**
	 * User Account listing
	 * @return [type] [description]
	 */
	public function action_index()
	{
		$this->session->delete('sub_link');

		$nav = 'nav_users';
		if ($this->session->get('nav_link') != $nav)
		{
			$this->session->set('nav_link', $nav);
		}

		$sort_fields = array(
			'date_created' => 'Date Created',
			'last_visit' => 'Last Visit Date',
			'name' => 'Name'
		);

		$sql = ORM::factory('Admin');

		if ($this->session->get('user_role_id') == 1)
		{
			$sql->where('role_id', '<=', 3)
				->or_where('role_id', '>', 7);
		}
		else if ($this->session->get('user_role_id') == 2)
		{
			$sql->where('role_id', '>=', 4)
				->where('role_id', '<=', 7);
		}

		if ($this->session->get('s_search_keyword'))
		{
			$search_keyword = $this->session->get('s_search_keyword');
			$sql->where_open()
				->where('username', 'LIKE', "%$search_keyword%")
				->or_where('name', 'LIKE', "%$search_keyword%")
				->or_where('email', 'LIKE', "%$search_keyword%")
				->where_close();

			$this->session->delete('s_search_keyword');
		}

		/*
		$page_query = clone $sql;
		$query = $page_query->find_all();
		$count = $query->count();
		// echo Database::instance()->last_query;
		$pagination = Pagination::factory(array(
		    'total_items'    => $count,
		    'view'           => 'pagination/floating',
		    'items_per_page' => 20
		));

		$page_links = $pagination->render();

		$sql->limit($pagination->items_per_page)
		    ->offset($pagination->offset);
		*/
		$sort_field = $this->session->get('s_sort_field') ? $this->session->get('s_sort_field') : 'date_created';
		$sort_by = $this->session->get('s_sort_by') ? $this->session->get('s_sort_by') : 'DESC';
		$list = $sql->order_by($sort_field, $sort_by)->find_all();

		$int_max_login_try = Kohana::$config->load('site')->get('max_login_try');
		$int_login_try = Kohana::$config->load('site')->get('login_try');

		$this->template->content = View::factory('admin/users/index');
		$this->template->content->sql= $list;
		// $this->template->content->pagination = $page_links;
		$this->template->content->sort_fields = $sort_fields;
		$this->template->content->int_max_login_try = $int_max_login_try;
		$this->template->content->int_login_try = $int_login_try;
	}

	/**
	 * Create User Account Form
	 * @return [type] [description]
	 */
	public function action_create()
	{
		$sub = 'sub_users';
		if ($this->session->get('sub_link') != $sub)
		{
			$this->session->set('sub_link', $sub);
		}

		$data = array(
			'segment_ids' => array()
		);

		$arr_roles = array();
		$arr_roles[''] = '';
		$qry = ORM::factory('Role');

		if ($this->session->get('user_role_id') == 1)
		{
			$qry->where('id', '<=', 3)
				->or_where('id', '>', 7);
		}
		else if ($this->session->get('user_role_id') == 2)
		{
			$qry->where('id', '>=', 4)
				->where('id', '<=', 7);
		}
		$sql = $qry->find_all();

		foreach ($sql as $res)
		{
			$arr_roles[$res->id] = $res->name;
		}

		$arr_categories = array();
		$default_class = array('class'=>'large');
		$this->template->content = View::factory('admin/users/form');
		$this->template->content->data = $data;
		$this->template->content->arr_roles = $arr_roles;
		$this->template->content->arr_categories = $arr_categories;
		$this->template->content->arr_segments = MyHelper::segments();
		$this->template->content->request_action = $this->request->action();
	}

	/**
	 * Save / Update User Information
	 * @return [type] [description]
	 */
	public function action_save()
	{
		if (HTTP_Request::POST == $this->request->method())
		{
			$token = Arr::get($this->request->post(), 'token');

			if (Security::check($token))
			{
				$user_role_id = $this->session->get('user_role_id');

				$name = HTML::chars(Arr::get($this->request->post(), 'name'));
				$username = HTML::chars(Arr::get($this->request->post(), 'username'));
				$password = HTML::chars(Arr::get($this->request->post(), 'password'));

				$passw = '';
				if ($password) {
					$passw = crypt($password, Cookie::$salt);
				}

				$email = HTML::chars(Arr::get($this->request->post(), 'email'));
				$office_phone = HTML::chars(Arr::get($this->request->post(), 'office_phone'));
				$role_id = Arr::get($this->request->post(), 'role_id');
				$rsql = ORM::factory('Role', $role_id);
				$role_name = $rsql->name;

				$category_ids = '';
				// User Admin is the one adding or updating the new user;
				if ($user_role_id == 2)
				{
					$category_ids = Arr::get($this->request->post(), 'category_ids');
					$category_ids = implode(",", $category_ids);
					$segment_ids = Arr::get($this->request->post(), 'segment_ids');
					$str_segment_ids = implode(",", $segment_ids);
				}

				$id = Arr::get($this->request->post(), 'id');

				$c_role_id = 0;
				if ($id)
				{
					$sql = ORM::factory('Admin', $id);
					$name = $sql->name;
					$username = $sql->username;
					$c_email = $sql->email;
					$c_office_phone = $sql->office_phone;
					$c_category_ids = $sql->category_ids;
					$c_role_id = $sql->role_id;
					$c_segment_ids = $sql->segment_ids;

					$arr_changed = array();
					// compare old value to the new value to check which field was updated
					if (strcmp($c_email, $email) != 0) {
						$arr_changed[] = 'email';
					}
					if (strcmp($c_office_phone, $office_phone) != 0) {
						$arr_changed[] = 'office_phone';
					}
					if (strcmp($c_category_ids, $category_ids) != 0) {
						$arr_changed[] = 'category_ids';
					}
					if (strcmp($c_role_id, $role_id) != 0 AND $role_id) {
						$arr_changed[] = 'role_id';
					}

					$action = 'Edit';
				}
				else
				{
					$sql = ORM::factory('Admin');
					$sql->date_created = date('Y-m-d H:i:s', time());

					$arr_changed = array();

					$action = 'Add';
				}

				if ($action == 'Add') {
					$sql->name = $name;
					$sql->username = $username;
					$sql->password = $passw;
				} else if ($action == 'Edit') {
					if ($password)
					{
						$sql->password = $passw;
					}
				}

				$sql->email = $email;
				$sql->office_phone = $office_phone;
				if ($role_id) {
					$sql->role_id = $role_id;
				}

				if ($user_role_id == 2)
				{
					$sql->category_ids = $category_ids;
					$sql->segment_ids = $str_segment_ids;
				}

				$sql->save();

				// for additional log info
				if ($user_role_id == 2)
				{
					if ($category_ids)
					{
						$categ = ORM::factory('Category')
							->where('id', 'IN', DB::expr('('.$category_ids.')'))
							->find_all();
						$categories = '';
						foreach ($categ as $res)
						{
							$categories .= ($categories ? ', ' : '').$res->name;
						}
					}

					$arr_segments = MyHelper::segments();
					if ($segment_ids)
					{
						$segments = '';
						foreach ($segment_ids as $k => $segment_id)
						{
							$segments .= ($segments ? ', ' : '').Arr::get($arr_segments, $segment_id);
						}
					}
				}

				$log_info = "User Name: ".$name."\r\nUser ID: ".$username;
				if ( ! $id)
				{
					$log_info .= "\r\nEmail: ".$email."\r\nOffice Phone: ".$office_phone;
					$log_info .= "\r\nRole: ".$role_name;
					if ($user_role_id == 2)
					{
						$log_info .= "\r\nCategories: ".$categories;
						$log_info .= "\r\nSegments: ".$segments;
					}
				}
				else
				{
					$log_info .= "\r\nEmail (Previous): ".$c_email."; Email (Current): ".$email;
					$log_info .= "\r\nOffice Phone (Previous): ".$c_office_phone."; Office Phone (Current): ".$office_phone;

					$rsql = ORM::factory('Role', $c_role_id);
					$c_role_name = $rsql->name;
					$log_info .= "\r\nRole (Previous): ".$c_role_name."; Role (Current): ".$role_name;
					if ($user_role_id == 2)
					{
						$c_categories = '';
						if ($c_category_ids)
						{
							$categ = ORM::factory('Category')
								->where('id', 'IN', DB::expr('('.$c_category_ids.')'))
								->find_all();
							foreach ($categ as $res)
							{
								$c_categories .= ($c_categories ? ', ' : '').$res->name;
							}
							$log_info .= "\r\nCategories (Previous): ".$c_categories."; Categories (Current): ".$categories;
						}
						$c_segments = '';
						$arr_segments = MyHelper::segments();
						if ($c_segment_ids)
						{
							$segment_ids = explode(",", $c_segment_ids);
							foreach ($segment_ids as $k => $segment_id)
							{
								$c_segments .= ($c_segments ? ', ' : '').Arr::get($arr_segments, $segment_id);
							}
							$log_info .= "\r\nSegments (Previous): ".$c_segments."; Segments (Current): ".$segments;
						}
					}
				}
				// end log info

				$log_action = "User";
				$checker_name = $this->session->get('checker_name');
				$log_description = "{$action} \"{$name}\" User Account";
				if ($user_role_id == 2)
				{
					$log_description .= "\r\nChecked By: ".$checker_name;
				}
				$log_description .="\r\n\r\n".$log_info;

				Activity::add($this->session->get('user_id'), $log_action, $log_description);

				$data = array(
					'name' => $name,
					'username' => $username,
					'email' => $email,
					'office_phone' => $office_phone,
					'role_id' => $role_id ? $role_id : $c_role_id,
					'category_ids' => $category_ids,
					'action' => strtolower($action),
					'changes' => $arr_changed,
					'date_added' => date('Y-m-d H:i:s', time())
				);

				$navlink = Arr::get($this->request->post(), 'navlink');
				if ($navlink == 'users')
				{
					$this->redirect('admin/users');
				} else if ($navlink == 'create') {
					$this->redirect('admin/users/create');
				} else if ($navlink == 'logs') {
					$this->redirect('admin/logs');
				} else if ($navlink == 'logout') {
					$this->redirect('admin/logout');
				} else if ($navlink == 'change_pw') {
					$this->redirect('admin/misc/change_pw');
				}

				if ($id)
				{
					$this->redirect('admin/users');
				} else {
					$this->session->set('flash_data', json_encode($data));

					$this->redirect('admin/users/msg_view');
				}
			}

			throw HTTP_Exception::factory(500, 'Access not allowed.');
		}

		throw HTTP_Exception::factory(404, 'URL not found!');
	}

	public function action_confirm_edit()
	{
		$this->auto_render = FALSE;
		if (HTTP_Request::POST == $this->request->method() AND $this->request->is_ajax())
		{
			$token = Arr::get($this->request->post(), 'token');

			if (Security::check($token))
			{
				$user_role_id = $this->session->get('user_role_id');
				$password = HTML::chars(Arr::get($this->request->post(), 'password'));

				if ($password) {
					$passw = crypt($password, Cookie::$salt);
				}

				$email = HTML::chars(Arr::get($this->request->post(), 'email'));
				$office_phone = HTML::chars(Arr::get($this->request->post(), 'office_phone'));
				$role_id = Arr::get($this->request->post(), 'role_id');

				$category_ids = '';
				$segment_ids = '';
				// User Admin is the one adding or updating the new user;
				if ($user_role_id == 2)
				{
					$category_ids = Arr::get($this->request->post(), 'category_ids');
					$category_ids = implode(",", $category_ids);
					$arr_segment_ids = Arr::get($this->request->post(), 'segment_ids');
					$segment_ids = implode(",", $arr_segment_ids);
				}

				$id = Arr::get($this->request->post(), 'id');

				if ($id)
				{
					$sql = ORM::factory('Admin', $id);
					$name = $sql->name;
					$username = $sql->username;
					$c_email = $sql->email;
					$c_office_phone = $sql->office_phone;
					$c_category_ids = $sql->category_ids;
					$c_role_id = $sql->role_id;
					$c_segment_ids = $sql->segment_ids;

					$arr_changed = array();
					// compare old value to the new value to check which field was updated
					if (strcmp($c_email, $email) != 0) {
						$arr_changed[] = 'email';
					}
					if (strcmp($c_office_phone, $office_phone) != 0) {
						$arr_changed[] = 'office_phone';
					}
					if (strcmp($c_category_ids, $category_ids) != 0) {
						$arr_changed[] = 'category_ids';
					}
					if (strcmp($c_role_id, $role_id) != 0 AND $role_id) {
						$arr_changed[] = 'role_id';
					}
					if (strcmp($c_segment_ids, $segment_ids) != 0) {
						$arr_changed[] = 'segment_ids';
					}

					$action = 'Edit';
					$form_data = $this->session->set('post_data', $this->request->post());

					$arr_data = array(
						'name' => $name,
						'username' => $username,
						'email' => $email,
						'office_phone' => $office_phone,
						'role_id' => $role_id,
						'category_ids' => $category_ids,
						'segment_ids' => $segment_ids,
						'action' => strtolower($action),
						'changes' => $arr_changed,
						'date_added' => date('Y-m-d H:i:s', time())
					);

					$qry = ORM::factory('Role', $role_id);
					$role = $qry->name;

					$categories = '';
					if ($category_ids)
					{
						$categ = ORM::factory('Category')
							->where('id', 'IN', DB::expr('('.$category_ids.')'))
							->find_all();
						foreach ($categ as $res)
						{
							$categories .= ($categories ? ', ' : '').$res->name;
						}
					}
					$segments = '';
					$arr_segments = MyHelper::segments();
					if ($segment_ids)
					{
						foreach ($arr_segment_ids as $k => $segment_id)
						{
							$segments .= ($segments ? ', ' : '').Arr::get($arr_segments, $segment_id);
						}
					}

					$arr_data['role'] = $role;
					$arr_data['categories'] = $categories;
					$arr_data['segments'] = $segments;

					$view = View::factory('admin/users/confirm_edit');
					$view->arr_data = $arr_data;

					$this->response->body($view);
				}
			}
		}
	}

	/**
	 * Display Edit Form
	 * @return [type] [description]
	 */
	public function action_edit()
	{
		$id = $this->request->param('id');

		$sql = ORM::factory('Admin', $id);

		$data = array(
			'id' => $sql->id,
			'name' => $sql->name,
			'username' => $sql->username,
			'password' => $sql->password,
			'email' => $sql->email,
			'office_phone' => $sql->office_phone,
			'category_ids' => $sql->category_ids ? explode(',', $sql->category_ids) : array(),
			'role_id' => $sql->role_id,
			'segment_ids' => $sql->segment_ids ? explode(",", $sql->segment_ids) : array()
		);

		$arr_categories = array();
		if ($this->session->get('user_role_id') == 2)
		{
			$role_id = $sql->role_id;
			$sql = ORM::factory('Category')
				->where(DB::expr('\',\'+').'roles'.DB::expr('+\',\''), 'LIKE', "%,$role_id,%")
				->find_all();

			foreach ($sql as $res)
			{
				$arr_categories[$res->id] = $res->name;
			}
		}

		$arr_roles = array();
		$arr_roles[''] = '';
		$qry = ORM::factory('Role');

		if ($this->session->get('user_role_id') == 1)
		{
			$qry->where('id', '<=', 3)
				->or_where('id', '>', 7);
		}
		else if ($this->session->get('user_role_id') == 2)
		{
			$qry->where('id', '>=', 4)
				->where('id', '<=', 7);
		}
		$sql = $qry->find_all();

		foreach ($sql as $res)
		{
			$arr_roles[$res->id] = $res->name;
		}

		$default_class = array('class'=>'large', 'disabled'=>'disabled');
		$this->template->content = View::factory('admin/users/form_edit');
		$this->template->content->data = $data;
		$this->template->content->arr_roles = $arr_roles;
		$this->template->content->arr_categories = $arr_categories;
		$this->template->content->arr_segments = MyHelper::segments();
		$this->template->content->default_class = $default_class;
		$this->template->content->request_action = $this->request->action();
	}

	/**
	 * Show the comment form when deleting a user account
	 * @return [type] [description]
	 */
	public function action_show_comment_form()
	{
		$this->auto_render = FALSE;

		if (HTTP_Request::POST == $this->request->method() AND $this->request->is_ajax())
		{
			$view = View::factory('admin/users/comment_form');

			$this->response->body($view);
		}
	}

	/**
	 * Process user account deletion
	 * @return [type] [description]
	 */
	public function action_delete()
	{
		$this->auto_render = FALSE;

		if (HTTP_Request::POST == $this->request->method())
		{
			$token = Arr::get($this->request->post(), 'token');

			if (Security::check($token))
			{
				$id = Arr::get($this->request->post(), 'id');
				// Special L1 User.  Should not be deleted
				if ($id == 1)
				{
					throw HTTP_Exception::factory(500, 'Unable to Delete.');
				}

				require_once Kohana::find_file('vendor', 'Html2Text/Html2Text');
				$comment = Arr::get($this->request->post(), 'comment');
				$html2text = new \Html2Text\Html2Text($comment);
				$comment = $html2text->get_text();

				$user_role_id = $this->session->get('user_role_id');

				$sql = ORM::factory('Admin', $id);
				if ($sql->loaded())
				{
					// $sql->deleted = 'Y';
					$name = $sql->name;
					$username = $sql->username;

					$sql->delete();

					$log_action = "User";
					$checker_name = $this->session->get('checker_name');
					$log_description = "Deleted \"{$name}\"";
					if ($user_role_id == 2)
					{
						$log_description .= "\r\nChecked By: ".$checker_name;
					}
					$log_description .= "\r\n\r\nUser ID: ".$username."\r\n\r\n{$comment}";

					Activity::add($this->session->get('user_id'), $log_action, $log_description);

					$this->redirect('admin/users');
				}
			}

			throw HTTP_Exception::factory(500, 'Access not allowed.');
		}

		throw HTTP_Exception::factory(404, 'URL not found!');
	}

	/**
	 * Get Roles assigned to specific category
	 * @return [type] [description]
	 */
	public function action_get_roles()
	{
		$this->auto_render = FALSE;

		if (HTTP_Request::POST == $this->request->method() AND $this->request->is_ajax())
		{
			$categ_id = Arr::get($this->request->post(), 'categ_id');
			$arr_roles = array();

			$sql = ORM::factory('Category', $categ_id);
			if ($sql->loaded())
			{
				$roles = $sql->roles;

				$sql = ORM::factory('Role')
					->where('id', 'IN', DB::expr('('.$roles.')'))
					->find_all();

				$arr_roles[] = '';
				foreach ($sql as $res)
				{
					$arr_roles[$res->id] = $res->name;
				}
			}

			echo json_encode($arr_roles);
		}
	}

	/**
	 * Get categories that has specific role requirements
	 * @return [type] [description]
	 */
	public function action_get_role_categories()
	{
		$this->auto_render = FALSE;

		$arr_categories = array();
		if (HTTP_Request::POST == $this->request->method() AND $this->request->is_ajax())
		{
			$role_id = Arr::get($this->request->post(), 'role_id');

			$sql = ORM::factory('Category')
				->where(DB::expr('\',\'+').'roles'.DB::expr('+\',\''), 'LIKE', "%,$role_id,%")
				->find_all();

			foreach ($sql as $res)
			{
				$arr_categories[$res->id] = $res->name;
			}
		}

		echo json_encode($arr_categories);
	}

	/**
	 * Display Message
	 * @return [type] [description]
	 */
	public function action_msg_view()
	{
		$view = View::factory('admin/users/msg_view');

		$flash_data = $this->session->get_once('flash_data');
		$arr_data = json_decode($flash_data, true);

		$category_ids = Arr::get($arr_data, 'category_ids');
		$role_id = Arr::get($arr_data, 'role_id');

		$qry = ORM::factory('Role', $role_id);
		$role = $qry->name;

		$categories = '';
		if ($category_ids)
		{
			$categ = ORM::factory('Category')
				->where('id', 'IN', DB::expr('('.$category_ids.')'))
				->find_all();
			$categories = '';
			foreach ($categ as $res)
			{
				$categories .= ($categories ? ', ' : '').$res->name;
			}
		}

		$arr_data['role'] = $role;
		$arr_data['categories'] = $categories;

		$view->arr_data = $arr_data;
		$this->template->content = $view;
	}

	/**
	 * Search Function
	 * @return [type] [description]
	 */
	public function action_search()
	{
		$this->auto_render = FALSE;

		if (HTTP_Request::POST == $this->request->method())
		{
			$search_keyword = HTML::chars(Arr::get($this->request->post(), 'search_keyword'));

			$this->session->set('s_search_keyword', $search_keyword);

			$this->redirect('admin/users');
		}
	}

	/**
	 * Sort list of users
	 * @return [type] [description]
	 */
	public function action_sort()
	{
		$this->auto_render = FALSE;

		if (HTTP_Request::POST == $this->request->method())
		{
			$sort_field = Arr::get($this->request->post(), 'sort_field');
			$sort_by = Arr::get($this->request->post(), 'sort_by');

			$this->session->set('s_sort_field', $sort_field);
			$this->session->set('s_sort_by', $sort_by);

			$this->redirect('admin/users');
		}
	}

	/**
	 * Check if username used is already registered in the system
	 * @return [type] [description]
	 */
	public function action_check_username()
	{
		$this->auto_render = FALSE;

		if ($this->request->post() AND $this->request->is_ajax())
		{
			$username = $this->request->post('username');

			$query = ORM::factory('Admin')
				->where('username', '=', $username)
				->find();

			if ($query->loaded())
			{
				echo json_encode(false);
			}
			else
			{
				echo json_encode(true);
			}
		}
	}

	public function action_reactivate()
	{
		$user_id = $this->request->param('id');

		$user = ORM::factory('Admin', $user_id);
		if ($user->loaded())
		{
			$date = date('F j, Y', time());
			$role = $user->role_id;
			$name = $user->name;
			$username = $user->username;
			$data = array(
				'action' => 'reactivate',
				'date' => $date,
				'role_id' => $role,
				'name' => $name
			);

			$max_login_try = $user->max_login_try;
			$int_max_login_try = Kohana::$config->load('site')->get('max_login_try');
			if ($max_login_try >= $int_max_login_try)
			{
				$user->max_login_try = 0;
			}

			$user->login_try = 0;
			$user->locked = '';
			$user->save();

			$user_role_id = $this->session->get('user_role_id');

			// For Activity
			$log_action = "User";
			$checker_name = $this->session->get('checker_name');
			$log_description = "User Account for \"{$name}\" has been re-activated";

			if ($user_role_id == 2)
			{
				$log_description .= "\r\nChecked By: ".$checker_name;
			}
			$log_description .= "\r\n\r\nUser ID: ".$username;

			// Log Activity
			Activity::add($this->session->get('user_id'), $log_action, $log_description);

			$this->session->set('flash_data', json_encode($data));

			$this->redirect('admin/users/msg_view');
		}
	}

	public function action_unsuspend()
	{
		$user_id = $this->request->param('id');

		$user = ORM::factory('Admin', $user_id);
		if ($user->loaded())
		{
			$date = date('F j, Y', time());
			$role = $user->role_id;
			$name = $user->name;
			$username = $user->username;
			$data = array(
				'action' => 'unsuspend',
				'date' => $date,
				'role_id' => $role,
				'name' => $name
			);

			$user->suspended = '';
			$user->last_visit = NULL;
			$user->save();

			$user_role_id = $this->session->get('user_role_id');

			// For Activity
			$log_action = "User";
			$checker_name = $this->session->get('checker_name');
			$log_description = "User Account for \"{$name}\" has been unsuspended";

			if ($user_role_id == 2)
			{
				$log_description .= "\r\nChecked By: ".$checker_name;
			}
			$log_description .= "\r\n\r\nUser ID: ".$username;

			// Log Activity
			Activity::add($this->session->get('user_id'), $log_action, $log_description);

			$this->session->set('flash_data', json_encode($data));

			$this->redirect('admin/users/msg_view');
		}
	}

	public function action_download()
	{
		$this->auto_render = FALSE;

		$sql = ORM::factory('Admin');

		if ($this->session->get('user_role_id') == 1)
		{
			$sql->where('role_id', '<=', 3)
				->or_where('role_id', '>', 7);
		}
		else if ($this->session->get('user_role_id') == 2)
		{
			$sql->where('role_id', '>=', 4)
				->where('role_id', '<=', 7);
		}

		$qry = $sql->order_by('date_created')
			->order_by('role_id')
			->find_all();

		if ($qry->count())
		{
			require_once Kohana::find_file('vendor', 'PHPExcel/PHPExcel');
			require_once Kohana::find_file('vendor', 'PHPExcel/PHPExcel/IOFactory');

			$objPHPExcel = new PHPExcel();
			$objPHPExcel->getProperties()->setTitle("EOL CMS User List");

			$objPHPExcel->setActiveSheetIndex(0)
				->setCellValue('A1', 'Name')
				->setCellValue('B1', 'Username')
				->setCellValue('C1', 'Email')
				->setCellValue('D1', 'Role')
				->setCellValue('E1', 'Segment')
				->setCellValue('F1', 'Category')
				->setCellValue('G1', 'Status');

			$ctr = 2;
			foreach ($qry as $res)
			{
				$name = html_entity_decode($res->name);
				$username = $res->username;
				$email = $res->email;
				$role = $res->role->name;
				$category_ids = $res->category_ids;
				$segment_ids = $res->segment_ids;
				$active = $res->active;
				$locked = $res->locked;
				$suspended = $res->suspended;

				if (($locked == 'Y' OR $suspended == 'Y') AND $active == 'Y')
				{
					$status = 'Inactive';
				}
				else
				{
					$status = 'Active';
				}

				$segments = '';
				$categories = '';
				if ($this->session->get('user_role_id') == 2)
				{
					if ($category_ids)
					{
						$categ = ORM::factory('Category')
							->where('id', 'IN', DB::expr('('.$category_ids.')'))
							->find_all();
						$categories = '';
						foreach ($categ as $res)
						{
							$categories .= ($categories ? ', ' : '').$res->name;
						}
						unset($categ);
					}

					$arr_segments = MyHelper::segments();
					if ($segment_ids)
					{
						$segment_ids = explode(",", $segment_ids);
						$segments = '';
						foreach ($segment_ids as $k => $segment_id)
						{
							$segments .= ($segments ? ', ' : '').Arr::get($arr_segments, $segment_id);
						}
					}
					unset($arr_segments);
				}

				$objPHPExcel->setActiveSheetIndex(0)
					->setCellValue('A'.$ctr, $name)
					->setCellValue('B'.$ctr, $username)
					->setCellValue('C'.$ctr, $email)
					->setCellValue('D'.$ctr, $role)
					->setCellValue('E'.$ctr, $segments)
					->setCellValue('F'.$ctr, $categories)
					->setCellValue('G'.$ctr, $status);

				$ctr++;
			}
		}

		$this->response->headers('Content-Type', 'application/vnd.ms-excel');
		$this->response->headers('Cache-Control', 'no-cache');
		$this->response->headers('Content-Disposition: attachment;filename="test.xlsx"');

		$objWriter = PHPExcel_IOFactory::createWriter($objPHPExcel, 'Excel2007');

		$objWriter->save('php://output');
		$objPHPExcel->disconnectWorksheets();
		unset($objPHPExcel);
		/*
		$fname = 'user_list.xls';
		$objWriter = PHPExcel_IOFactory::createWriter($objPHPExcel, 'Excel2007');
		//$objWriter->save(str_replace('.php', '.xlsx', __FILE__));
		$objWriter->save(DOCROOT.'media/'.$fname);

		header("location: ".URL::site('media/'.$fname));
		exit;
		*/
	}
}