<?php defined('SYSPATH') or die('No direct script access.');

class MyHelper {

	/**
	 * Get list of Content Type
	 * @param  string $ctypes comma separted value of content type id
	 * @return array         id=>name of content type
	 */
	public static function get_content_type($ctypes)
	{
		$arr_data = array();

		$qry = ORM::factory('ContentType')
			->where('id', 'IN', DB::expr('('.$ctypes.')'))
			->find_all();

		foreach ($qry as $res)
		{
			$arr_data[$res->id] = $res->name;
		}

		return $arr_data;
	}

	public static function content_types()
	{
		$qry = ORM::factory('ContentType')
			->find_all();
		foreach ($qry as $res)
		{
			$arr_data[$res->id] = $res->name;
		}

		return $arr_data;
	}

	public static function segments()
	{
		$arr_data = array(
			1 => 'Personal',
			/*
			2 => 'Preferred Banking',
			3 => 'Private Banking',
			*/
			4 => 'Business',
			5 => 'Overseas Filipino'
		);

		return $arr_data;
	}

	public static function page_status()
	{
		// Different label for different roles in some page status
		$arr_status = array(
			'pending-check' => 'Pending for Checking',
			'pending-concur' => array(
				4 => 'Checked',
				5 => 'Checked',
				6 => 'Pending for Checking'
			),
			'draft' => 'Draft',
			'rework' => 'For Rework',
			'rework-archive' => 'For Rework',
			'denied' => 'Denied',
			'concurred' => 'Concurred',
			'published' => 'Published',
			'archived' => 'Archived',
			'pending-publish' => array(
				4 => 'Checked',
				5 => 'Checked',
				6 => 'Checked',
				7 => 'Pending for Publishing'
			),
			'pending-archive' => array(
				4 => 'Checked',
				5 => 'Checked',
				6 => 'Checked',
				7 => 'Pending for Archiving'
			),
			'request-archive' => array(
				4 => 'Pending for Checking',
				5 => 'Pending for Checking',
				6 => 'Published',
				7 => 'Published'
			),
			'check-archive' => array(
				4 => 'Checked',
				5 => 'Checked',
				6 => 'Pending for Checking',
				7 => 'Published'
			),
			'denied-archive' => 'Denied Archive',
			'cancel-rework-archive' => 'Cancelled Rework Archive'
		);

		return $arr_status;
	}

	public static function page_status_search($needle, $haystack, $user_type)
	{
		$results = array();
		foreach($haystack as $key => $value)
		{
	     	$current_key = $key;

	     	if (is_array($value))
	     	{
	     		foreach ($value as $nkey => $nvalue)
	     		{
	     			if (strtolower($needle) == strtolower($nvalue) AND $user_type == $nkey)
	     			{
	     				$results[] = $current_key;
	     			}
	     		}
	     	}
	     	else if(strtolower($needle) === strtolower($value))
	     	{
	       	$results[] = $current_key;
	     	}
	   }

	   return $results;
	}

	// Generate control no
	public static function generate_control_no()
	{
		$current_date = date('Y-m-d', time());
		$qry = DB::select()->from('sequence_no')->where('d8', '=', $current_date)->as_object()->execute();
		if ($qry->count())
		{
			$ctr = $qry->current();
			$sid = $ctr->id;
			$seq = $ctr->seq;
			$sequence = $seq + 1;
			DB::update('sequence_no')->set(array('seq'=>$sequence))->where('id', '=', $sid)->where('d8', '=', $current_date)->execute();
		}
		else
		{
			$sequence = 1;
			DB::insert('sequence_no', array('d8', 'seq'))->values(array($current_date, 1))->execute();
		}

		$sequence_no = str_pad($sequence, 4, "0", STR_PAD_LEFT);
		$control_no = date('Y-md-Hi', time()).'-'.$sequence_no;

		return $control_no;
	}

	public static function role_options($user_role_id)
	{
		$arr_roles = array();
		$arr_roles[''] = '';
		$qry = ORM::factory('Role');
		if ($user_role_id == 1)
		{
			$qry->where('id', '<=', 3);
		}
		else if ($user_role_id == 2)
		{
			$qry->where('id', '>=', 4);
		}

		$sql = $qry->find_all();
		foreach ($sql as $res)
		{
			$arr_roles[$res->id] = $res->name;
		}

		return $arr_roles;
	}

	public static function field_type($section)
	{
		if (strstr($section, 'title'))
		{
			$type = 'text';
		}
		else if (strstr($section, 'link'))
		{
			$type = 'link';
		}
		else if (strstr($section, 'embed'))
		{
			$type = 'embed';
		}
		else if (strstr($section, 'body'))
		{
			$type = 'textarea';
		}
		else if (strstr($section, 'image'))
		{
			$type = 'file_img';
		}
		else if (strstr($section, 'doc'))
		{
			$type = 'file_doc';
		}

		return $type;
	}

	public static function regular_fields()
	{
		$fields = array(
			'title',
			'body1',
			'body2',
			'body3',
			'body4',
			'body5',
			'link1',
			'link2',
			'link3',
			'link4',
			'link5',
			'embed1',
			'embed2',
			'embed3',
			'embed4',
			'embed5',
			'image1',
			'image2',
			'image3',
			'image4',
			'image5',
			'doc1',
			'doc2',
			'doc3',
			'doc4',
			'doc5'
		);

		return $fields;
	}

	public static function document_history($page_id, $user_id, $description)
	{
		$user = ORM::factory('Admin', $user_id);
		if ($user->loaded())
		{
			$sql = ORM::factory('DocumentHistory');
			$sql->page_id = $page_id;
			$sql->user_id = $user_id;
			$sql->user_name = $user->name;
			$sql->role = $user->role->name;
			$sql->description = HTML::chars($description);
			$sql->date_added = date('Ymd h:i:s A', time());
			$sql->save();
		}
	}

	// check user role and article status if they can interact with it
	// 4 = Maker, 5 = Checker, 6 = Checker2, 7 = Publisher
	public static function check_role_status($user_role_id, $page_status)
	{
		$arr_allowed = array();
		switch ($user_role_id)
		{
			case 4:
				$arr_allowed = array(
					'draft',
					'rework',
					'published',
					'archived',
					'rework-archive'
				);
				break;
			case 5:
				$arr_allowed = array(
					'pending-check',
					'request-archive'
				);
				break;
			case 6:
				$arr_allowed = array(
					'pending-concur',
					'check-archive'
				);
				break;
			case 7:
				$arr_allowed = array(
					'pending-publish',
					'pending-archive'
				);
				break;
		}

		$result = in_array($page_status, $arr_allowed);

		return $result;
	}

	/**
	 * check if there's an assiged checker 2 for a particular category
	 * @param  [type]  $content_type_id [content type id]
	 * @param  [type]  $value   		[value to check against the user account (category)]
	 * @return boolean          		[description]
	 */
	public static function has_checker2($content_type_id, $value)
	{
		// find a user account with a role of checker 2 that has access to category
		$sql = ORM::factory('Admin')
			->where('role_id', '=', 6)
			->where(DB::expr('\',\'+').'category_ids'.DB::expr('+\',\''), 'LIKE', "%,$value,%");
		/*
		if ($content_type_id == 2 OR $content_type_id == 5) // splash banner or client update
		{
			if ($value)
			{
				$arr_segments = explode(",", $value);

				$sql->where_open();
				foreach ($arr_segments as $s => $segment_id)
				{
				$sql->or_where(DB::expr('\',\'+').'segment_ids'.DB::expr('+\',\''), 'LIKE', "%,$segment_id,%");
				}
				$sql->where_close();
			}
		}
		else
		{
			if ($value)
			{
				$sql->where(DB::expr('\',\'+').'category_ids'.DB::expr('+\',\''), 'LIKE', "%,$value,%");
			}
		}
		*/

		$qry = $sql->find_all();

		return $qry->count();
	}

	/**
	 * Get id of Sub Category1 based on Category ID
	 * @param  [type] $category_id [description]
	 * @return [type]              [description]
	 */
	public static function get_sub_category1($category_id)
	{
		$sql = ORM::factory('Navtree')
			->where('category_id', '=', $category_id)
			->where('level', '=', 1)
			->find_all();

		$arr_sub_category1 = array();
		foreach ($sql as $res)
		{
			$arr_sub_category1[] = $res->id;
		}

		return $arr_sub_category1;
	}

	public static function banner_source($category_id, array $arr_sub_category1)
	{
		$arr_banner_source = array();

		$sub_category1 = '';
		if (count($arr_sub_category1) > 0)
		{
			$sub_category1 = implode(",", $arr_sub_category1);
		}

		$current_date = date('Y-m-d', time());

		$qry = ORM::factory('Page')
			->where('start_date', '<=', $current_date)
			->where('end_date', '>=', $current_date)
			->where_open()
			->where('status', '=', 'published')
			->or_where('status', '=', 'request-archive')
			->or_where('status', '=', 'pending-archive')
			->or_where('status', '=', 'check-archive')
			->or_where('status', '=', 'rework-archive')
			->where_close()
			->where('content_type_id', '=', 3)
			->where_open()
			->where('category_id', '=', $category_id);
		if ($sub_category1)
		{
			$qry->or_where('sub_category1', 'IN', DB::expr('('.$sub_category1.')'));
		}
		$sql = $qry->where_close()
			->order_by('id')
			->find_all();

		foreach ($sql as $res)
		{
			$arr_banner_source[$res->id] = $res->title;
		}

		return $arr_banner_source;
	}

	/**
	 * Generate Navigation Menu
	 * @param  [type] $segment_id [description]
	 * @return [type]             [description]
	 */
	public static function menu($segment_id)
	{
		$sid = 'personal';
		$cls = ' class="personal"';
		$segment = URL::title(Arr::get(MyHelper::segments(), $segment_id));

		$navstring = '<ul'.$cls.'><li><a href="'.URL::site('home/menu/personal').'">Home</a>';
		if ($segment_id == 2)
		{
			$cls = ' class="preferred"';
			$navstring = '<ul'.$cls.'><li><a href="'.URL::site('home/menu/preferred-banking').'">Home</a>';
			$sid = 'preferred';
		}
		else if ($segment_id == 3)
		{
			$cls = ' class="private"';
			$navstring = '<ul'.$cls.'><li><a href="'.URL::site('home/menu/private-banking').'">Home</a>';
			$sid = 'private';
		}
		else if ($segment_id == 4)
		{
			$cls = ' class="bussiness"';
			$navstring = '<ul'.$cls.'><li><a href="'.URL::site('home/menu/business').'">Home</a>';
			$sid = 'business';
		}
		else if ($segment_id == 5)
		{
			$cls = ' class="filip"';
			$navstring = '<ul'.$cls.'><li><a href="'.URL::site('home/menu/overseas-filipino').'">Home</a>';
			$sid = 'filip';
		}

		$sql = ORM::factory('Menu')
			->where('segment_id', '=', $segment_id)
			->order_by('sort')
			->find_all();

		foreach ($sql as $res)
		{
			$menu_id = $res->id;
			$menu_name = $res->action;
			$categories = $res->categories;
			$menu = URL::title($menu_name);

			$categ = '';
			if ($categories)
			{
				$csql = ORM::factory('Category')
					->where('id', 'IN', DB::expr('('.$categories.')'))
					->find_all();
				if ($csql->count())
				{
					$categ = '<ul>';
					foreach ($csql as $c)
					{
						$cid = $c->id;
						$category_name = $c->name;
						$category = URL::title($category_name);

						// check subcategory 1
						// check if this has sub category
						$csql1 = ORM::factory('Navtree')
							->where('category_id', '=', $cid)
							->where('level', '=', 1)
							->find_all();

						$sub_1 = '';
						if ($csql1->count())
						{
							$sub_1 = '<ul>';
							foreach ($csql1 as $c1)
							{
								$cid_1 = $c1->id;
								$subcategory_1 = $c1->name;
								$subcategory1 = URL::title($subcategory_1);

								// check subcategory 2
								// check if this has sub category
								$csql2 = ORM::factory('Navtree')
									->where('category_id', '=', $cid)
									->where('parent_id', '=', $cid_1)
									->where('level', '=', 2)
									->find_all();

								$sub_2 = '';
								if ($csql2->count())
								{
									$sub_2 = '<ul>';
									foreach ($csql2 as $c2)
									{
										$cid_2 = $c2->id;
										$subcategory_2 = $c2->name;
										$subcategory2 = URL::title($subcategory_2);

										// check subcategory 3
										// check if this has sub category
										$csql3 = ORM::factory('Navtree')
											->where('category_id', '=', $cid)
											->where('parent_id', '=', $cid_2)
											->where('level', '=', 3)
											->find_all();

										$sub_3 = '';
										if ($csql3->count())
										{
											$sub_3 = '<ul>';
											foreach ($csql3 as $c3)
											{
												$cid_3 = $c3->id;
												$subcategory_3 = $c3->name;
												$subcategory3 = URL::title($subcategory_3);
												// $navlink = 'p/'.$segment.'/'.URL::title($menu).'/'.URL::title($categname).'/'.URL::title($categname_2).'/'.URL::title($categname_3).'/'.URL::title($categname_4);

												$check_navstring = MyHelper::check_navstring($cid, $cid_1, $cid_2, $cid_3);
												if ($check_navstring)
												{
													$navlink = URL::site('p/3/'.$cid_3.'/'.$subcategory3);
													$navstyle = '';
												}
												else
												{
													$navlink = '#';
													$navstyle = ' style="cursor:default;color:#333;"';
												}

												$sub_3 .= '<li><a href="'.$navlink.'"'.$navstyle.' class="submenu" rel="'.$sid.'|menu_'.$segment_id.'_'.$menu_id.'">'.$subcategory_3.'</a></li>';
											}
											$sub_3 .= '</ul>';
										}
										unset($csql3);

										// $navlink = $sub_2 ? '#' : 'p/'.$segment.'/'.URL::title($menu).'/'.URL::title($categname).'/'.URL::title($categname_2).'/'.URL::title($categname_3);
										// $navlink = $sub_2 ? '#' : URL::site('p/'.$cid_3.'/'.$l4);

										$check_navstring = MyHelper::check_navstring($cid, $cid_1, $cid_2, NULL);
										if ($check_navstring)
										{
											$navlink = URL::site('p/2/'.$cid_2.'/'.$subcategory2);
											$navstyle = '';
										}
										else
										{
											$navlink = '#';
											$navstyle = ' style="cursor:default;color:#333;"';
										}

										$sub_2 .= '<li><a href="'.$navlink.'"'.$navstyle.' class="submenu" rel="'.$sid.'|menu_'.$segment_id.'_'.$menu_id.'">'.$subcategory_2.'</a>'.$sub_3.'</li>';
									}
									$sub_2 .= '</ul>';
								}
								unset($csql2);

								// $navlink = $sub_1 ? '#' : 'p/'.$segment.'/'.URL::title($menu).'/'.URL::title($categname).'/'.URL::title($categname_2);
								// $navlink = $sub_1 ? '#' : URL::site('p/'.$cid_2.'/'.$l3);

								$check_navstring = MyHelper::check_navstring($cid, $cid_1, NULL, NULL);
								if ($check_navstring)
								{
									$navlink = URL::site('p/1/'.$cid_1.'/'.$subcategory1);
									$navstyle = '';
								}
								else
								{
									$navlink = '#';
									$navstyle = ' style="cursor:default;color:#333;"';
								}

								$sub_1 .= '<li><a href="'.$navlink.'"'.$navstyle.' class="submenu" rel="'.$sid.'|menu_'.$segment_id.'_'.$menu_id.'">'.$subcategory_1.'</a>'.$sub_2.'</li>';
							}
							$sub_1 .= '</ul>';
						}
						unset($csql1);

						// $navlink = $sub ? '#' : 'p/'.$segment.'/'.URL::title($menu).'/'.URL::title($categname);

						// $navlink = $sub ? '#' : URL::site('p/'.$cid.'/'.$l2);

						$check_navstring = MyHelper::check_navstring($cid, NULL, NULL, NULL);
						if ($check_navstring)
						{
							$navlink = URL::site('p/0/'.$cid.'/'.$category);
							$navstyle = '';
						}
						else
						{
							$navlink = '#';
							$navstyle = ' style="cursor:default;color:#333;"';
						}

						$categ .= '<li><a href="'.$navlink.'"'.$navstyle.' class="submenu" rel="'.$sid.'|menu_'.$segment_id.'_'.$menu_id.'">'.$category_name.'</a>'.$sub_1.'</li>';
					}
					$categ .= '</ul>';
				}
				unset($csql);
			}

			$elm = array();
			if (Session::instance()->get('menu_active'))
			{
				$menu_active = Session::instance()->get('menu_active');
				$elm = explode("|", $menu_active);
			}

			$active = '';
			if (count($elm))
			{
				if ($elm[1] == 'menu_'.$segment_id.'_'.$menu_id)
				{
					$active = ' class="active"';
				}
			}

			$navstring .= '<li id="menu_'.$segment_id.'_'.$menu_id.'"'.$active.'><a href="#">'.$menu_name.'</a>'.$categ.'</li>';
		}

		$navstring .= '</ul>';

		return $navstring;
	}

	public static function check_navstring($category_id, $sub_category1=NULL, $sub_category2=NULL, $sub_category3=NULL)
	{
		$current_date = date('Y-m-d', time());

		$sql = ORM::factory('Page')
			->where('start_date', '<=', $current_date)
			->where('end_date', '>=', $current_date)
			->where_open()
			->where_open()
			->where('status', '=', 'published')
			->or_where('status', '=', 'request-archive')
			->or_where('status', '=', 'pending-archive')
			->or_where('status', '=', 'check-archive')
			->or_where('status', '=', 'rework-archive')
			->where_close()
			->where('content_type_id', '=', 1)
			->or_where('content_type_id', '>', 7)
			->where_close()
			->where('category_id', '=', $category_id);

		if ($sub_category1 > 0)
		{
			$sql->where('sub_category1', '=', $sub_category1);
		} else {
			$sql->where_open()
				->where('sub_category1', 'IS', NULL)
				->or_where('sub_category1', '=', 0)
				->where_close();
		}
		if ($sub_category2 > 0)
		{
			$sql->where('sub_category2', '=', $sub_category2);
		} else {
			$sql->where_open()
				->where('sub_category2', 'IS', NULL)
				->or_where('sub_category2', '=', 0)
				->where_close();
		}
		if ($sub_category3 > 0)
		{
			$sql->where('sub_category3', '=', $sub_category3);
		} else {
			$sql->where_open()
				->where('sub_category3', 'IS', NULL)
				->or_where('sub_category3', '=', 0)
				->where_close();
		}

		$qry = $sql->find_all();

		if ($qry->count())
		{
			return true;
		} else {
			return false;
		}
	}

	public static function search_navstring($category_id, $sub_category1=NULL, $sub_category2=NULL, $sub_category3=NULL, $protocol=false)
	{
		if ($sub_category1 AND $sub_category2 AND $sub_category3)
		{
			$csql = ORM::factory('Navtree')
				->where('category_id', '=', $category_id)
				->where('parent_id', '=', $sub_category2)
				->where('id', '=', $sub_category3)
				->where('level', '=', 3)
				->find();
			$category_name = URL::title($csql->name);

			$navlink = URL::site('p/3/'.$sub_category3.'/'.$category_name, $protocol);
		}
		else if ($sub_category1 AND $sub_category2 AND (! $sub_category3 OR is_null($sub_category3)))
		{
			$csql = ORM::factory('Navtree')
				->where('category_id', '=', $category_id)
				->where('parent_id', '=', $sub_category1)
				->where('id', '=', $sub_category2)
				->where('level', '=', 2)
				->find();
			$category_name = URL::title($csql->name);
			$navlink = URL::site('p/2/'.$sub_category2.'/'.$category_name, $protocol);
		}
		else if ($sub_category1 AND (! $sub_category2 OR is_null($sub_category2)) AND (! $sub_category3 OR is_null($sub_category3)))
		{
			$csql = ORM::factory('Navtree')
				->where('category_id', '=', $category_id)
				->where('id', '=', $sub_category1)
				->where('level', '=', 1)
				->find();
			$category_name = URL::title($csql->name);
			$navlink = URL::site('p/1/'.$sub_category1.'/'.$category_name, $protocol);
		}
		else if ($category_id AND (! $sub_category1 OR is_null($sub_category1)) AND (! $sub_category2 OR is_null($sub_category2)) AND (! $sub_category3 OR is_null($sub_category3)))
		{
			$csql = ORM::factory('Category')
				->where('id', '=', $category_id)
				->find();
			$category_name = URL::title($csql->name);
			$navlink = URL::site('p/0/'.$category_id.'/'.$category_name, $protocol);
		}

		return $navlink;
	}

	/**
	 * Product Category
	 * For Promo
	 * @return [type]             [description]
	 */
	public static function product_category()
	{
		/*
		$arr_data = array(
			'' => '',
			1 => 'Credit Cards',
			2 => 'ATM/Debit Cards',
			3 => 'Deposits',
			4 => 'Loans',
			5 => 'Prepaid Cards',
			6 => 'Remittance',
			7 => 'BPI 24/7 Banking',
			8 => 'Branch Banking',
			9 => 'Investments',
			10 => 'Preferred Banking'
		);
		*/

		$sql = ORM::factory('PromoCategory')
			->find_all();
		$arr_data = array();
		$arr_data[''] = '';
		foreach ($sql as $res)
		{
			$arr_data[$res->id] = $res->name;
		}

		return $arr_data;
	}

	public static function check_user_login($user_id)
	{
		$bootout = 'N';

		$qry = DB::select()
			->from('sessions')
			->where('user_id', '=', $user_id)
			->where('session_id', '=', Session::instance()->id())
			->as_object()
			->execute();
		if ($qry->count())
		{
			$sql = $qry->current();
			$bootout = $sql->bootout;
		}

		return $bootout;
	}

	public static function login_box($segment_id)
	{
		if ( ! $segment_id)
		{
			$segment_id = 1;
		}

		$view = View::factory('login_box');
		$view->segment_id = $segment_id;

		return $view;
	}

	public static function top_bottom_links()
	{
		$arr_data = array(
			'related_sites' => 'Related Sites',
			'about_us' => 'About Us',
			'apply_now' => 'Apply Now',
			'promos' => 'Promos',
			'rates' => 'Rates',
			'locator' => 'Locator',
			'contact_us' => 'Contact Us',
			'product_services' => 'Product & Services',
			'market_watcher' => 'Market Watcher',
			'careers' => 'Careers',
			'security' => 'Security',
			'preferred_banking' => 'Preferred Banking',
			'private_banking' => 'Private Banking'
		);

		return $arr_data;
	}

	public static function last_login()
	{
		if (Session::instance()->get('last_login') != null)
		{
			$last_login = date('M j, Y g:i a', strtotime(Session::instance()->get('last_login')));
		}
		else
		{
			$last_login = '';
		}

		return $last_login;
	}

	public static function login_log($arr_data)
	{
		// $contents = '';

		$filename = DOCROOT.'/media/logs/log.php';
		/*
		if (file_exists($filename))
		{
			$handle = fopen($filename, "rb");
			$contents = fread($handle, filesize($filename));
			fclose($handle);
		}

		$data = 'Time: '.date('M j, Y g:i:s a', time()).' Account: '.Arr::get($arr_data, 'uname').'/'.Arr::get($arr_data, 'pword');
		$newdata = Kohana::FILE_SECURITY.' ?>'.PHP_EOL.($contents ? $contents.PHP_EOL : '').$data;
		$fp = fopen($filename, 'wb');
		$contents = fwrite($fp, $newdata);
		fclose($handle);
		*/
		$data = 'Time: '.date('M j, Y g:i:s a', time()).' Account: '.Arr::get($arr_data, 'uname').'/'.Arr::get($arr_data, 'pword').PHP_EOL;
		file_put_contents($filename, $data, FILE_APPEND);
	}

	public static function close_tags($html)
	{
		$single_tags = array('meta', 'img', 'br', 'link', 'area', 'input', 'hr', 'col', 'param', 'base');

		preg_match_all('~<([a-z0-9]+)(?: .*)?(?<![/|/ ])>~iU', $html, $result);
		$openedtags = $result[1];
		preg_match_all('~</([a-z0-9]+)>~iU', $html, $result);
		$closedtags = $result[1];
		$len_opened = count($openedtags);

		if (count($closedtags) == $len_opened)
		{
			return $html;
		}

		$openedtags = array_reverse($openedtags);

		for ($i = 0; $i < $len_opened; $i++)
		{
			if ( ! in_array($openedtags[$i], $single_tags))
			{
				if (($key = array_search($openedtags[$i], $closedtags)) !== FALSE)
				{
					unset($closedtags[$key]);
				}
				else
				{
					$html .= '</'.$openedtags[$i].'>';
				}
			}
		}

		return $html;
	}
}